@@ -5,95 +5,77 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * This program is distributed under the terms of the Nmap Public Source   *
+ * License (NPSL). The exact license text applying to a particular Nmap    *
+ * release or source code control revision is contained in the LICENSE     *
+ * file distributed with that version of Nmap or source code control       *
+ * revision. More Nmap copyright/legal information is available from       *
+ * https://nmap.org/book/man-legal.html, and further information on the    *
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+ * header summarizes some key points from the Nmap license, but is no      *
+ * substitute for the actual license text.                                 *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * Nmap is generally free for end users to download and use themselves,    *
+ * including commercial use. It is available from https://nmap.org.        *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * The Nmap license generally prohibits companies from using and           *
+ * redistributing Nmap in commercial products, but we sell a special Nmap  *
+ * OEM Edition with a more permissive license and special features for     *
+ * this purpose. See https://nmap.org/oem/                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * If you have received a written Nmap license agreement or contract       *
+ * stating terms other than these (such as an Nmap OEM license), you may   *
+ * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * The official Nmap Windows builds include the Npcap software             *
+ * (https://npcap.com) for packet capture and transmission. It is under    *
+ * separate license terms which forbid redistribution without special      *
+ * permission. So the official Nmap Windows builds may not be              *
+ * redistributed without special permission (such as an Nmap OEM           *
+ * license).                                                               *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * and add new features.  You are highly encouraged to submit your         *
+ * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+ * for possible incorporation into the main distribution. Unless you       *
+ * specify otherwise, it is understood that you are offering us very       *
+ * broad rights to use your submissions as described in the Nmap Public    *
+ * Source License Contributor Agreement. This is important because we      *
+ * fund the project by selling licenses with various terms, and also       *
+ * because the inability to relicense code has caused devastating          *
+ * problems for other Free Software projects (such as KDE and NASM).       *
  *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * The free version of Nmap is distributed in the hope that it will be     *
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+ * indemnification and commercial support are all available through the    *
+ * Npcap OEM program--see https://nmap.org/oem/                            *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.h 22066 2011-01-27 21:49:15Z david $ */
+/* $Id: NmapOps.h 38353 2022-02-18 17:38:46Z dmiller $ */
 
-#include "nmap.h"
-#include "global_structures.h"
-#include "output.h"
+#ifndef NMAP_OPS_H
+#define NMAP_OPS_H
+
+#include "nmap.h" /* MAX_DECOYS */
+#include "scan_lists.h"
+#include "output.h" /* LOG_NUM_FILES */
+#include <nbase.h>
+#include <nsock.h>
 #include <string>
+#include <map>
+#include <vector>
+
+struct FingerPrintDB;
+struct FingerMatch;
 
 class NmapOps {
  public:
@@ -107,17 +89,17 @@ class NmapOps {
   /* Returns 0 for success, nonzero if no source has been set or any other
      failure */
   int SourceSockAddr(struct sockaddr_storage *ss, size_t *ss_len);
+  /* Returns a const pointer to the source address if set, or NULL if unset. */
+  const struct sockaddr_storage *SourceSockAddr() const;
   /* Note that it is OK to pass in a sockaddr_in or sockaddr_in6 casted
      to sockaddr_storage */
   void setSourceSockAddr(struct sockaddr_storage *ss, size_t ss_len);
 
 // The time this obj. was instantiated   or last ReInit()ed.
   const struct timeval *getStartTime() { return &start_time; }
-  // Number of milliseconds since getStartTime().  The current time is an
+  // Number of seconds since getStartTime().  The current time is an
   // optional argument to avoid an extra gettimeofday() call.
-  int TimeSinceStartMS(const struct timeval *now=NULL); 
-  struct in_addr v4source();
-  const struct in_addr *v4sourceip();
+  float TimeSinceStart(const struct timeval *now=NULL);
 
 
 
@@ -129,17 +111,18 @@ class NmapOps {
      It does not currently cover cases such as TCP SYN ping scan which
      can go either way based on whether the user is root or IPv6 is
      being used.  It will return false in those cases where a RawScan
-     is not neccessarily used. */
+     is not necessarily used. */
   bool RawScan();
   void ValidateOptions(); /* Checks that the options given are
-                             reasonable and consistant.  If they aren't, the
+                             reasonable and consistent.  If they aren't, the
                              function may bail out of Nmap or make small
-		             adjustments (quietly or with a warning to the
-		             user). */
+                             adjustments (quietly or with a warning to the
+                             user). */
   int isr00t;
   /* Whether we have pcap functions (can be false on Windows). */
   bool have_pcap;
-  int debugging;
+  u8 debugging;
+  bool resuming;
 
 #define PACKET_SEND_NOPREF 1
 #define PACKET_SEND_ETH_WEAK 2
@@ -171,23 +154,25 @@ class NmapOps {
   void setVersionTrace(bool vt) { vTrace = vt;  }
   bool openOnly() { return open_only; }
   void setOpenOnly(bool oo) { open_only = oo; }
-  int verbose;
+  u8 verbose;
   /* The requested minimum packet sending rate, or 0.0 if unset. */
   float min_packet_send_rate;
   /* The requested maximum packet sending rate, or 0.0 if unset. */
   float max_packet_send_rate;
   /* The requested auto stats printing interval, or 0.0 if unset. */
   float stats_interval;
-  int randomize_hosts;
-  int spoofsource; /* -S used */
-  int fastscan;
+  bool randomize_hosts;
+  bool randomize_ports;
+  bool spoofsource; /* -S used */
+  bool fastscan;
   char device[64];
   int ping_group_sz;
-  int nogcc; /* Turn off group congestion control with --nogcc */
-  int generate_random_ips; /* -iR option */
+  bool nogcc; /* Turn off group congestion control with --nogcc */
+  bool generate_random_ips; /* -iR option */
   FingerPrintDB *reference_FPs; /* Used in the new OS scan system. */
-  u16 magic_port;
-  unsigned short magic_port_set; /* Was this set by user? */
+  std::vector<FingerMatch> os_labels_ipv6;
+  u16 magic_port; /* The source port set by -g or --source-port. */
+  bool magic_port_set; /* Was this set by user? */
 
   /* Scan timing/politeness issues */
   int timing_level; // 0-5, corresponding to Paranoid, Sneaky, Polite, Normal, Aggressive, Insane
@@ -204,7 +189,7 @@ class NmapOps {
 
   /* These functions retrieve and set the Round Trip Time timeouts, in
    milliseconds.  The set versions do extra processing to insure sane
-   values and to adjust each other to insure consistance (e.g. that
+   values and to adjust each other to insure consistence (e.g. that
    max is always at least as high as min) */
   int maxRttTimeout() { return max_rtt_timeout; }
   int minRttTimeout() { return min_rtt_timeout; }
@@ -213,7 +198,7 @@ class NmapOps {
   void setMinRttTimeout(int rtt);
   void setInitialRttTimeout(int rtt);
   void setMaxRetransmissions(int max_retransmit);
-  int getMaxRetransmissions() { return max_retransmissions; }
+  unsigned int getMaxRetransmissions() { return max_retransmissions; }
 
   /* Similar functions for Host group size */
   int minHostGroupSz() { return min_host_group_sz; }
@@ -235,14 +220,14 @@ class NmapOps {
   /* Returns the full path or URL that should be printed in the XML
      output xml-stylesheet element.  Returns NULL if the whole element
      should be skipped */
-  char *XSLStyleSheet() { return xsl_stylesheet; }
+  char *XSLStyleSheet();
 
   /* Sets the spoofed MAC address */
   void setSpoofMACAddress(u8 *mac_data);
   /* Gets the spoofed MAC address, but returns NULL if it hasn't been set */
   const u8 *spoofMACAddress() { return spoof_mac_set? spoof_mac : NULL; }
 
-  int max_ips_to_scan; // Used for Random input (-iR) to specify how 
+  unsigned int max_ips_to_scan; // Used for Random input (-iR) to specify how
                        // many IPs to try before stopping. 0 means unlimited.
   int extra_payload_length; /* These two are for --data-length op */
   char *extra_payload;
@@ -252,57 +237,61 @@ class NmapOps {
   bool open_only;
 
   int scanflags; /* if not -1, this value should dictate the TCP flags
-		    for the core portscaning routine (eg to change a
-		    FIN scan into a PSH scan.  Sort of a hack, but can
-		    be very useful sometimes. */
+                    for the core portscanning routine (eg to change a
+                    FIN scan into a PSH scan.  Sort of a hack, but can
+                    be very useful sometimes. */
 
-  int defeat_rst_ratelimit; /* Solaris 9 rate-limits RSTs so scanning is very
+  bool defeat_rst_ratelimit; /* Solaris 9 rate-limits RSTs so scanning is very
             slow against it. If we don't distinguish between closed and filtered ports,
             we can get the list of open ports very fast */
 
-  struct in_addr resume_ip; /* The last IP in the log file if user 
-			       requested --restore .  Otherwise 
-			       restore_ip.s_addr == 0.  Also 
-			       target_struct_get will eventually set it 
-			       to 0. */
+  bool defeat_icmp_ratelimit; /* If a host rate-limits ICMP responses, then scanning
+            is very slow against it. This option prevents Nmap to adjust timing
+            when it changes the port's state because of ICMP response, as the latter
+            might be rate-limited. Doing so we can get scan results faster. */
+
+  struct sockaddr_storage resume_ip; /* The last IP in the log file if user
+                               requested --restore .  Otherwise
+                               resume_ip.ss_family == AF_UNSPEC.  Also
+                               Target::next_target will eventually set it
+                               to AF_UNSPEC. */
 
   // Version Detection Options
-  int override_excludeports;
+  bool override_excludeports;
   int version_intensity;
 
-  struct in_addr decoys[MAX_DECOYS];
-  int osscan_limit; /* Skip OS Scan if no open or no closed TCP ports */
-  int osscan_guess;   /* Be more aggressive in guessing OS type */
+  struct sockaddr_storage decoys[MAX_DECOYS];
+  bool osscan_limit; /* Skip OS Scan if no open or no closed TCP ports */
+  bool osscan_guess;   /* Be more aggressive in guessing OS type */
   int numdecoys;
   int decoyturn;
-  int osscan;
-  int servicescan;
+  bool osscan;
+  bool servicescan;
   int pingtype;
   int listscan;
-  int allowall;
   int fragscan; /* 0 or MTU (without IPv4 header size) */
   int ackscan;
   int bouncescan;
   int connectscan;
   int finscan;
   int idlescan;
+  char* idleProxy; /* The idle host used to "Proxy" an idle scan */
   int ipprotscan;
   int maimonscan;
   int nullscan;
-  int rpcscan;
   int synscan;
   int udpscan;
   int sctpinitscan;
   int sctpcookieechoscan;
   int windowscan;
   int xmasscan;
-  int noresolve;
-  int noportscan;
-  int append_output; /* Append to any output files rather than overwrite */
+  bool noresolve;
+  bool noportscan;
+  bool append_output; /* Append to any output files rather than overwrite */
   FILE *logfd[LOG_NUM_FILES];
   FILE *nmap_stdout; /* Nmap standard output */
   int ttl; // Time to live
-  int badsum;
+  bool badsum;
   char *datadir;
   /* A map from abstract data file names like "nmap-services" and "nmap-os-db"
      to paths which have been requested by the user. nmap_fetchfile will return
@@ -313,20 +302,44 @@ class NmapOps {
      Only files that were actually read should be in this map. */
   std::map<std::string, std::string> loaded_data_files;
   bool mass_dns;
-  int resolve_all;
+  bool always_resolve;
+  bool resolve_all;
+  bool unique;
   char *dns_servers;
-  bool log_errors;
+
+  /* Do IPv4 ARP or IPv6 ND scan of directly connected Ethernet hosts, even if
+     non-ARP host discovery options are used? This is normally more efficient,
+     not only because ARP/ND scan is faster, but because we need the MAC
+     addresses provided by ARP or ND scan in order to do IP-based host discovery
+     anyway. But when a network uses proxy ARP, all hosts will appear to be up
+     unless you do an IP host discovery on them. This option is true by default. */
+  bool implicitARPPing;
+
+  // If true, write <os><osclass/><osmatch/></os> as in xmloutputversion 1.03
+  // rather than <os><osmatch><osclass/></osmatch></os> as in 1.04 and later.
+  bool deprecated_xml_osclass;
+
   bool traceroute;
   bool reason;
   bool adler32;
+  FILE *excludefd;
+  char *exclude_spec;
+  FILE *inputfd;
+  char *portlist; /* Ports list specified by user */
+  char *exclude_portlist; /* exclude-ports list specified by user */
+
+  nsock_proxychain proxy_chain;
+  bool discovery_ignore_rst; /* host discovery should not consider TCP RST packet responses as a live asset */
 
 #ifndef NOLUA
-  int script;
+  bool script;
   char *scriptargs;
-  int scriptversion;
-  int scripttrace;
-  int scriptupdatedb;
+  char *scriptargsfile;
+  bool scriptversion;
+  bool scripttrace;
+  bool scriptupdatedb;
   bool scripthelp;
+  double scripttimeout;
   void chooseScripts(char* argument);
   std::vector<std::string> chosenScripts;
 #endif
@@ -338,8 +351,8 @@ class NmapOps {
   int ipopt_lasthop;	// offset in ipoptions where is space for targets ip for source/strict routing
 
   // Statistics Options set in nmap.cc
-  int numhosts_scanned;
-  int numhosts_up;
+  unsigned int numhosts_scanned;
+  unsigned int numhosts_up;
   int numhosts_scanning;
   stype current_scantype;
   bool noninteractive;
@@ -350,21 +363,23 @@ class NmapOps {
   int max_rtt_timeout;
   int min_rtt_timeout;
   int initial_rtt_timeout;
-  int max_retransmissions;
+  unsigned int max_retransmissions;
   unsigned int max_tcp_scan_delay;
   unsigned int max_udp_scan_delay;
   unsigned int max_sctp_scan_delay;
   unsigned int min_host_group_sz;
   unsigned int max_host_group_sz;
   void Initialize();
-  int addressfamily; /*  Address family:  AF_INET or AF_INET6 */  
+  int addressfamily; /*  Address family:  AF_INET or AF_INET6 */
   struct sockaddr_storage sourcesock;
   size_t sourcesocklen;
   struct timeval start_time;
   bool pTrace; // Whether packet tracing has been enabled
   bool vTrace; // Whether version tracing has been enabled
+  bool xsl_stylesheet_set;
   char *xsl_stylesheet;
   u8 spoof_mac[6];
   bool spoof_mac_set;
 };
-  
+
+#endif
