@@ -2,126 +2,58 @@
  * ncat_digest.c -- HTTP Digest authentication handling.                   *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2019 Insecure.Com LLC ("The Nmap  *
+ * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
  * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
+ * This program is distributed under the terms of the Nmap Public Source   *
+ * License (NPSL). The exact license text applying to a particular Nmap    *
+ * release or source code control revision is contained in the LICENSE     *
+ * file distributed with that version of Nmap or source code control       *
+ * revision. More Nmap copyright/legal information is available from       *
+ * https://nmap.org/book/man-legal.html, and further information on the    *
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+ * header summarizes some key points from the Nmap license, but is no      *
+ * substitute for the actual license text.                                 *
  *                                                                         *
- * o Integrates source code from Covered Software.                         *
+ * Nmap is generally free for end users to download and use themselves,    *
+ * including commercial use. It is available from https://nmap.org.        *
  *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
+ * The Nmap license generally prohibits companies from using and           *
+ * redistributing Nmap in commercial products, but we sell a special Nmap  *
+ * OEM Edition with a more permissive license and special features for     *
+ * this purpose. See https://nmap.org/oem/                                 *
  *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
+ * If you have received a written Nmap license agreement or contract       *
+ * stating terms other than these (such as an Nmap OEM license), you may   *
+ * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, we are happy to help.  As mentioned above, we also *
- * offer an alternative license to integrate Nmap into proprietary         *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing support and updates.  They also fund the continued         *
- * development of Nmap.  Please email sales@nmap.com for further           *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
+ * The official Nmap Windows builds include the Npcap software             *
+ * (https://npcap.com) for packet capture and transmission. It is under    *
+ * separate license terms which forbid redistribution without special      *
+ * permission. So the official Nmap Windows builds may not be              *
+ * redistributed without special permission (such as an Nmap OEM           *
+ * license).                                                               *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
  * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
+ * and add new features.  You are highly encouraged to submit your         *
+ * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+ * for possible incorporation into the main distribution. Unless you       *
+ * specify otherwise, it is understood that you are offering us very       *
+ * broad rights to use your submissions as described in the Nmap Public    *
+ * Source License Contributor Agreement. This is important because we      *
+ * fund the project by selling licenses with various terms, and also       *
+ * because the inability to relicense code has caused devastating          *
+ * problems for other Free Software projects (such as KDE and NASM).       *
  *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
+ * The free version of Nmap is distributed in the hope that it will be     *
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+ * indemnification and commercial support are all available through the    *
+ * Npcap OEM program--see https://nmap.org/oem/                            *
  *                                                                         *
  ***************************************************************************/
 
@@ -145,7 +77,7 @@
 #include "ncat.h"
 #include "http.h"
 
-#include <openssl/md5.h>
+#include <openssl/evp.h>
 #include <openssl/rand.h>
 
 /* What's a good length for this? I think it exists only to prevent us from
@@ -201,14 +133,19 @@ int http_digest_init_secret(void)
     return 0;
 }
 
+#if OPENSSL_API_LEVEL < 10100
+#define EVP_MD_CTX_new EVP_MD_CTX_create
+#define EVP_MD_CTX_free EVP_MD_CTX_destroy
+#endif
 static char *make_nonce(const struct timeval *tv)
 {
     char *buf = NULL;
     size_t size = 0, offset = 0;
-    MD5_CTX md5;
-    unsigned char hashbuf[MD5_DIGEST_LENGTH];
-    char hash_hex[MD5_DIGEST_LENGTH * 2 + 1];
+    EVP_MD_CTX *md5;
+    unsigned char hashbuf[EVP_MAX_MD_SIZE];
+    char hash_hex[EVP_MAX_MD_SIZE * 2 + 1];
     char time_buf[32];
+    unsigned int hash_size = 0;
 
     /* Crash if someone forgot to call http_digest_init_secret. */
     if (!secret_initialized)
@@ -216,13 +153,13 @@ static char *make_nonce(const struct timeval *tv)
 
     Snprintf(time_buf, sizeof(time_buf), "%lu.%06lu",
         (long unsigned) tv->tv_sec, (long unsigned) tv->tv_usec);
-
-    MD5_Init(&md5);
-    MD5_Update(&md5, secret, sizeof(secret));
-    MD5_Update(&md5, ":", 1);
-    MD5_Update(&md5, time_buf, strlen(time_buf));
-    MD5_Final(hashbuf, &md5);
-    enhex(hash_hex, hashbuf, sizeof(hashbuf));
+    md5 = EVP_MD_CTX_new();
+    EVP_DigestInit_ex(md5, EVP_md5(), NULL);
+    EVP_DigestUpdate(md5, secret, sizeof(secret));
+    EVP_DigestUpdate(md5, ":", 1);
+    EVP_DigestUpdate(md5, time_buf, strlen(time_buf));
+    EVP_DigestFinal_ex(md5, hashbuf, &hash_size);
+    enhex(hash_hex, hashbuf, hash_size);
 
     strbuf_sprintf(&buf, &size, &offset, "%s-%s", time_buf, hash_hex);
 
@@ -231,51 +168,54 @@ static char *make_nonce(const struct timeval *tv)
 
 /* Arguments are assumed to be non-NULL, with the exception of nc and cnonce,
    which may be garbage only if qop == QOP_NONE. */
-static void make_response(char buf[MD5_DIGEST_LENGTH * 2 + 1],
+static void make_response(char buf[EVP_MAX_MD_SIZE * 2 + 1],
     const char *username, const char *realm, const char *password,
     const char *method, const char *uri, const char *nonce,
     enum http_digest_qop qop, const char *nc, const char *cnonce)
 {
-    char HA1_hex[MD5_DIGEST_LENGTH * 2 + 1], HA2_hex[MD5_DIGEST_LENGTH * 2 + 1];
-    unsigned char hashbuf[MD5_DIGEST_LENGTH];
-    MD5_CTX md5;
+    char HA1_hex[EVP_MAX_MD_SIZE * 2 + 1], HA2_hex[EVP_MAX_MD_SIZE * 2 + 1];
+    unsigned char hashbuf[EVP_MAX_MD_SIZE];
+    EVP_MD_CTX *md5;
+    unsigned int hash_size = 0;
+    const EVP_MD *md = EVP_md5();
 
     /* Calculate H(A1). */
-    MD5_Init(&md5);
-    MD5_Update(&md5, username, strlen(username));
-    MD5_Update(&md5, ":", 1);
-    MD5_Update(&md5, realm, strlen(realm));
-    MD5_Update(&md5, ":", 1);
-    MD5_Update(&md5, password, strlen(password));
-    MD5_Final(hashbuf, &md5);
-    enhex(HA1_hex, hashbuf, sizeof(hashbuf));
+    md5 = EVP_MD_CTX_new();
+    EVP_DigestInit_ex(md5, md, NULL);
+    EVP_DigestUpdate(md5, username, strlen(username));
+    EVP_DigestUpdate(md5, ":", 1);
+    EVP_DigestUpdate(md5, realm, strlen(realm));
+    EVP_DigestUpdate(md5, ":", 1);
+    EVP_DigestUpdate(md5, password, strlen(password));
+    EVP_DigestFinal_ex(md5, hashbuf, &hash_size);
+    enhex(HA1_hex, hashbuf, hash_size);
 
     /* Calculate H(A2). */
-    MD5_Init(&md5);
-    MD5_Update(&md5, method, strlen(method));
-    MD5_Update(&md5, ":", 1);
-    MD5_Update(&md5, uri, strlen(uri));
-    MD5_Final(hashbuf, &md5);
-    enhex(HA2_hex, hashbuf, sizeof(hashbuf));
+    EVP_DigestInit_ex(md5, md, NULL);
+    EVP_DigestUpdate(md5, method, strlen(method));
+    EVP_DigestUpdate(md5, ":", 1);
+    EVP_DigestUpdate(md5, uri, strlen(uri));
+    EVP_DigestFinal_ex(md5, hashbuf, &hash_size);
+    enhex(HA2_hex, hashbuf, hash_size);
 
     /* Calculate response. */
-    MD5_Init(&md5);
-    MD5_Update(&md5, HA1_hex, strlen(HA1_hex));
-    MD5_Update(&md5, ":", 1);
-    MD5_Update(&md5, nonce, strlen(nonce));
+    EVP_DigestInit_ex(md5, md, NULL);
+    EVP_DigestUpdate(md5, HA1_hex, strlen(HA1_hex));
+    EVP_DigestUpdate(md5, ":", 1);
+    EVP_DigestUpdate(md5, nonce, strlen(nonce));
     if (qop == QOP_AUTH) {
-        MD5_Update(&md5, ":", 1);
-        MD5_Update(&md5, nc, strlen(nc));
-        MD5_Update(&md5, ":", 1);
-        MD5_Update(&md5, cnonce, strlen(cnonce));
-        MD5_Update(&md5, ":", 1);
-        MD5_Update(&md5, "auth", strlen("auth"));
+        EVP_DigestUpdate(md5, ":", 1);
+        EVP_DigestUpdate(md5, nc, strlen(nc));
+        EVP_DigestUpdate(md5, ":", 1);
+        EVP_DigestUpdate(md5, cnonce, strlen(cnonce));
+        EVP_DigestUpdate(md5, ":", 1);
+        EVP_DigestUpdate(md5, "auth", strlen("auth"));
     }
-    MD5_Update(&md5, ":", 1);
-    MD5_Update(&md5, HA2_hex, strlen(HA2_hex));
-    MD5_Final(hashbuf, &md5);
+    EVP_DigestUpdate(md5, ":", 1);
+    EVP_DigestUpdate(md5, HA2_hex, strlen(HA2_hex));
+    EVP_DigestFinal_ex(md5, hashbuf, &hash_size);
 
-    enhex(buf, hashbuf, sizeof(hashbuf));
+    enhex(buf, hashbuf, hash_size);
 }
 
 /* Extract the issuance time from a nonce (without checking other aspects of
@@ -326,7 +266,7 @@ char *http_digest_proxy_authorization(const struct http_challenge *challenge,
        varying client nonce count. */
     static const u32 nc = 0x00000001;
 
-    char response_hex[MD5_DIGEST_LENGTH * 2 + 1];
+    char response_hex[EVP_MAX_MD_SIZE * 2 + 1];
     unsigned char cnonce[CNONCE_LENGTH];
     char cnonce_buf[CNONCE_LENGTH * 2 + 1];
     char nc_buf[8 + 1];
@@ -334,7 +274,8 @@ char *http_digest_proxy_authorization(const struct http_challenge *challenge,
     size_t size = 0, offset = 0;
     enum http_digest_qop qop;
 
-    if (challenge->scheme != AUTH_DIGEST || challenge->realm == NULL
+    if (challenge->scheme != AUTH_DIGEST
+        || challenge->realm == NULL
         || challenge->digest.nonce == NULL
         || challenge->digest.algorithm != ALGORITHM_MD5)
         return NULL;
@@ -389,7 +330,7 @@ int http_digest_check_credentials(const char *username, const char *realm,
     const char *password, const char *method,
     const struct http_credentials *credentials)
 {
-    char response_hex[MD5_DIGEST_LENGTH * 2 + 1];
+    char response_hex[EVP_MAX_MD_SIZE * 2 + 1];
     struct timeval tv;
     char *nonce;
 
@@ -398,7 +339,8 @@ int http_digest_check_credentials(const char *username, const char *realm,
         || credentials->u.digest.realm == NULL
         || credentials->u.digest.nonce == NULL
         || credentials->u.digest.uri == NULL
-        || credentials->u.digest.response == NULL) {
+        || credentials->u.digest.response == NULL
+        || credentials->u.digest.algorithm != ALGORITHM_MD5) {
         return 0;
     }
     if (credentials->u.digest.qop != QOP_NONE && credentials->u.digest.qop != QOP_AUTH)
