@@ -2,93 +2,62 @@
  * ncat_proxy.c -- HTTP proxy server.                                      *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * This program is distributed under the terms of the Nmap Public Source   *
+ * License (NPSL). The exact license text applying to a particular Nmap    *
+ * release or source code control revision is contained in the LICENSE     *
+ * file distributed with that version of Nmap or source code control       *
+ * revision. More Nmap copyright/legal information is available from       *
+ * https://nmap.org/book/man-legal.html, and further information on the    *
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+ * header summarizes some key points from the Nmap license, but is no      *
+ * substitute for the actual license text.                                 *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * Nmap is generally free for end users to download and use themselves,    *
+ * including commercial use. It is available from https://nmap.org.        *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@insecure.com for     *
- * further information.                                                    *
+ * The Nmap license generally prohibits companies from using and           *
+ * redistributing Nmap in commercial products, but we sell a special Nmap  *
+ * OEM Edition with a more permissive license and special features for     *
+ * this purpose. See https://nmap.org/oem/                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * If you have received a written Nmap license agreement or contract       *
+ * stating terms other than these (such as an Nmap OEM license), you may   *
+ * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * The official Nmap Windows builds include the Npcap software             *
+ * (https://npcap.com) for packet capture and transmission. It is under    *
+ * separate license terms which forbid redistribution without special      *
+ * permission. So the official Nmap Windows builds may not be              *
+ * redistributed without special permission (such as an Nmap OEM           *
+ * license).                                                               *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify otherwise) *
- * that you are offering the Nmap Project (Insecure.Com LLC) the           *
- * unlimited, non-exclusive right to reuse, modify, and relicense the      *
- * code.  Nmap will always be available Open Source, but this is important *
- * because the inability to relicense code has caused devastating problems *
- * for other Free Software projects (such as KDE and NASM).  We also       *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
+ * and add new features.  You are highly encouraged to submit your         *
+ * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+ * for possible incorporation into the main distribution. Unless you       *
+ * specify otherwise, it is understood that you are offering us very       *
+ * broad rights to use your submissions as described in the Nmap Public    *
+ * Source License Contributor Agreement. This is important because we      *
+ * fund the project by selling licenses with various terms, and also       *
+ * because the inability to relicense code has caused devastating          *
+ * problems for other Free Software projects (such as KDE and NASM).       *
  *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * The free version of Nmap is distributed in the hope that it will be     *
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+ * indemnification and commercial support are all available through the    *
+ * Npcap OEM program--see https://nmap.org/oem/                            *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: ncat_proxy.c 28192 2012-03-01 06:53:35Z fyodor $ */
+/* $Id: ncat_proxy.c 38353 2022-02-18 17:38:46Z dmiller $ */
 
 #include "base64.h"
 #include "http.h"
@@ -104,7 +73,8 @@
 /* SIG_CHLD handler */
 static void proxyreaper(int signo)
 {
-    while (waitpid(-1, NULL, WNOHANG) > 0);
+    while (waitpid(-1, NULL, WNOHANG) > 0)
+        ;
 }
 #endif
 
@@ -158,6 +128,7 @@ int ncat_http_server(void)
     int listen_socket[NUM_LISTEN_ADDRS];
     socklen_t sslen;
     union sockaddr_u conn;
+    unsigned int num_sockets;
 
 #ifndef WIN32
     Signal(SIGCHLD, proxyreaper);
@@ -182,16 +153,29 @@ int ncat_http_server(void)
     init_fdlist(&listen_fdlist, num_listenaddrs);
 
     /* Listen on each address, set up lists for select */
+    num_sockets = 0;
     for (i = 0; i < num_listenaddrs; i++) {
-        listen_socket[i] = do_listen(SOCK_STREAM, IPPROTO_TCP, &listenaddrs[i]);
+        listen_socket[num_sockets] = do_listen(SOCK_STREAM, IPPROTO_TCP, &listenaddrs[i]);
+        if (listen_socket[num_sockets] == -1) {
+            if (o.debug > 0)
+                logdebug("do_listen(\"%s\"): %s\n", inet_ntop_ez(&listenaddrs[i].storage, sizeof(listenaddrs[i].storage)), socket_strerror(socket_errno()));
+            continue;
+        }
 
-        /* make us not block on accepts in wierd cases. See ncat_listen.c:209 */
-        unblock_socket(listen_socket[i]);
+        /* make us not block on accepts in weird cases. See ncat_listen.c:209 */
+        unblock_socket(listen_socket[num_sockets]);
 
         /* setup select sets and max fd */
-        FD_SET(listen_socket[i], &listen_fds);
-        add_fd(&listen_fdlist, listen_socket[i]);
+        checked_fd_set(listen_socket[num_sockets], &listen_fds);
+        add_fd(&listen_fdlist, listen_socket[num_sockets]);
 
+        num_sockets++;
+    }
+    if (num_sockets == 0) {
+        if (num_listenaddrs == 1)
+            bye("Unable to open listening socket on %s: %s", inet_ntop_ez(&listenaddrs[0].storage, sizeof(listenaddrs[0].storage)), socket_strerror(socket_errno()));
+        else
+            bye("Unable to open any listening sockets.");
     }
 
     for (;;) {
@@ -204,18 +188,22 @@ int ncat_http_server(void)
         if (o.debug > 1)
             logdebug("selecting, fdmax %d\n", listen_fdlist.fdmax);
         read_fds = listen_fds;
+
         int fds_ready = fselect(listen_fdlist.fdmax + 1, &read_fds, NULL, NULL, NULL);
 
         if (o.debug > 1)
             logdebug("select returned %d fds ready\n", fds_ready);
 
-        for (i = 0; i <= listen_fdlist.fdmax && fds_ready >0; i++) {
+        if (fds_ready == 0)
+            bye("Idle timeout expired (%d ms).", o.idletimeout);
+
+        for (i = 0; i <= listen_fdlist.fdmax && fds_ready > 0; i++) {
             /* Loop through descriptors until there is something ready */
-            if (!FD_ISSET(i, &read_fds))
+            if (!checked_fd_isset(i, &read_fds))
                 continue;
 
             /* Check each listening socket */
-            for (j = 0; j < num_listenaddrs; j++) {
+            for (j = 0; j < num_sockets; j++) {
                 if (i == listen_socket[j]) {
                     fds_ready--;
                     c = accept(i, &conn.sockaddr, &sslen);
@@ -231,7 +219,7 @@ int ncat_http_server(void)
                         continue;
                     }
                     if (o.debug > 1)
-                            logdebug("forking handler for %d\n", i);
+                        logdebug("forking handler for %d\n", i);
                     fork_handler(i, c);
                 }
             }
@@ -424,7 +412,7 @@ static int handle_connect(struct socket_buffer *client_sock,
 {
     union sockaddr_u su;
     size_t sslen = sizeof(su.storage);
-    int maxfd, s;
+    int maxfd, s, rc;
     char *line;
     size_t len;
     fd_set m, r;
@@ -435,11 +423,14 @@ static int handle_connect(struct socket_buffer *client_sock,
         return 400;
     }
     if (o.debug > 1)
-        logdebug("CONNECT to %s:%hu.\n", request->uri.host, request->uri.port);
+        logdebug("CONNECT to %s:%d.\n", request->uri.host, request->uri.port);
 
-    if (!resolve(request->uri.host, request->uri.port, &su.storage, &sslen, o.af)) {
-        if (o.debug)
-            logdebug("Can't resolve name %s.\n", request->uri.host);
+    rc = resolve(request->uri.host, request->uri.port, &su.storage, &sslen, o.af);
+    if (rc != 0) {
+        if (o.debug) {
+            logdebug("Can't resolve name \"%s\": %s.\n",
+                request->uri.host, gai_strerror(rc));
+        }
         return 504;
     }
 
@@ -459,15 +450,15 @@ static int handle_connect(struct socket_buffer *client_sock,
     line = socket_buffer_remainder(client_sock, &len);
     if (send(s, line, len, 0) < 0) {
         if (o.debug)
-            logdebug("Error sending %u leftover bytes: %s.\n", len, strerror(errno));
+            logdebug("Error sending %lu leftover bytes: %s.\n", (unsigned long) len, strerror(errno));
         Close(s);
         return 0;
     }
 
     maxfd = client_sock->fdn.fd < s ? s : client_sock->fdn.fd;
     FD_ZERO(&m);
-    FD_SET(client_sock->fdn.fd, &m);
-    FD_SET(s, &m);
+    checked_fd_set(client_sock->fdn.fd, &m);
+    checked_fd_set(s, &m);
 
     errno = 0;
 
@@ -481,7 +472,7 @@ static int handle_connect(struct socket_buffer *client_sock,
 
         zmem(buf, sizeof(buf));
 
-        if (FD_ISSET(client_sock->fdn.fd, &r)) {
+        if (checked_fd_isset(client_sock->fdn.fd, &r)) {
             do {
                 do {
                     len = fdinfo_recv(&client_sock->fdn, buf, sizeof(buf));
@@ -497,7 +488,7 @@ static int handle_connect(struct socket_buffer *client_sock,
             } while (fdinfo_pending(&client_sock->fdn));
         }
 
-        if (FD_ISSET(s, &r)) {
+        if (checked_fd_isset(s, &r)) {
             do {
                 len = recv(s, buf, sizeof(buf), 0);
             } while (len == -1 && socket_errno() == EINTR);
@@ -529,7 +520,7 @@ static int handle_method(struct socket_buffer *client_sock,
     union sockaddr_u su;
     size_t sslen = sizeof(su.storage);
     int code;
-    int s;
+    int s, rc;
 
     if (strcmp(request->uri.scheme, "http") != 0) {
         if (o.verbose)
@@ -542,9 +533,12 @@ static int handle_method(struct socket_buffer *client_sock,
         return 400;
     }
 
-    if (!resolve(request->uri.host, request->uri.port, &su.storage, &sslen, o.af)) {
-        if (o.debug)
-            logdebug("Can't resolve name %s:%d.\n", request->uri.host, request->uri.port);
+    rc = resolve(request->uri.host, request->uri.port, &su.storage, &sslen, o.af);
+    if (rc != 0) {
+        if (o.debug) {
+            logdebug("Can't resolve name %s:%d: %s.\n",
+                request->uri.host, request->uri.port, gai_strerror(rc));
+        }
         return 504;
     }
 
@@ -596,7 +590,7 @@ static int do_transaction(struct http_request *request,
        request) if it cannot determine the length of the message, or with 411
        (length required) if it wishes to insist on receiving a valid
        Content-Length." */
-    if (strcmp(request->method, "POST") == 0 && request->content_length == 0) {
+    if (strcmp(request->method, "POST") == 0 && !request->content_length_set) {
         if (o.debug)
             logdebug("POST request with no Content-Length.\n");
         return 400;
@@ -617,7 +611,7 @@ static int do_transaction(struct http_request *request,
     if (request->uri.port == -1 || request->uri.port == 80)
         n = Snprintf(buf, sizeof(buf), "%s", request->uri.host);
     else
-        n = Snprintf(buf, sizeof(buf), "%s:%hu", request->uri.host, request->uri.port);
+        n = Snprintf(buf, sizeof(buf), "%s:%d", request->uri.host, request->uri.port);
     if (n < 0 || n >= sizeof(buf)) {
         /* Request Entity Too Large. */
         return 501;
@@ -707,14 +701,16 @@ static int do_transaction(struct http_request *request,
     /* If the Content-Length is 0, read until the connection is closed.
        Otherwise read until the Content-Length. At this point it's too late to
        return our own error code so return 0 in case of any error. */
-    while (response.content_length == 0
+    while (!response.content_length_set
         || response.bytes_transferred < response.content_length) {
-        size_t remaining = response.content_length - response.bytes_transferred;
         size_t count;
 
         count = sizeof(buf);
-        if (response.content_length > 0 && remaining < count)
-            count = remaining;
+        if (response.content_length_set) {
+            size_t remaining = response.content_length - response.bytes_transferred;
+            if (remaining < count)
+                count = remaining;
+        }
         n = socket_buffer_read(server_sock, buf, count);
         if (n <= 0)
             break;
@@ -824,12 +820,13 @@ static int check_auth(const struct http_request *request,
 
         /* Split up the proxy auth argument. */
         proxy_auth = Strdup(o.proxy_auth);
-        username = strtok(proxy_auth, ":");
-        password = strtok(NULL, ":");
+        username = proxy_auth;
+        password = strchr(proxy_auth, ':');
         if (password == NULL) {
             free(proxy_auth);
             return 0;
         }
+        *password++ = '\0';
         ret = http_digest_check_credentials(username, "Ncat", password,
             request->method, credentials);
         free(proxy_auth);
