@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2020 Insecure.Com LLC ("The Nmap  *
+ * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
  * Project"). Nmap is also a registered trademark of the Nmap Project.     *
  *                                                                         *
  * This program is distributed under the terms of the Nmap Public Source   *
@@ -13,9 +13,9 @@
  * file distributed with that version of Nmap or source code control       *
  * revision. More Nmap copyright/legal information is available from       *
  * https://nmap.org/book/man-legal.html, and further information on the    *
- * NPSL license itself can be found at https://nmap.org/npsl. This header  *
- * summarizes some key points from the Nmap license, but is no substitute  *
- * for the actual license text.                                            *
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+ * header summarizes some key points from the Nmap license, but is no      *
+ * substitute for the actual license text.                                 *
  *                                                                         *
  * Nmap is generally free for end users to download and use themselves,    *
  * including commercial use. It is available from https://nmap.org.        *
@@ -23,14 +23,14 @@
  * The Nmap license generally prohibits companies from using and           *
  * redistributing Nmap in commercial products, but we sell a special Nmap  *
  * OEM Edition with a more permissive license and special features for     *
- * this purpose. See https://nmap.org/oem                                  *
+ * this purpose. See https://nmap.org/oem/                                 *
  *                                                                         *
  * If you have received a written Nmap license agreement or contract       *
  * stating terms other than these (such as an Nmap OEM license), you may   *
  * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
  * The official Nmap Windows builds include the Npcap software             *
- * (https://npcap.org) for packet capture and transmission. It is under    *
+ * (https://npcap.com) for packet capture and transmission. It is under    *
  * separate license terms which forbid redistribution without special      *
  * permission. So the official Nmap Windows builds may not be              *
  * redistributed without special permission (such as an Nmap OEM           *
@@ -55,11 +55,11 @@
  * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
  * indemnification and commercial support are all available through the    *
- * Npcap OEM program--see https://nmap.org/oem.                            *
+ * Npcap OEM program--see https://nmap.org/oem/                            *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_ftp.cc 38078 2020-10-02 16:12:22Z dmiller $ */
+/* $Id: nmap_ftp.cc 38353 2022-02-18 17:38:46Z dmiller $ */
 #include "nmap.h"
 #include "nmap_ftp.h"
 #include "output.h"
@@ -321,14 +321,12 @@ void bounce_scan(Target *target, u16 *portarray, int numports,
                 }
                 else {
                   recvbuf[res] = '\0';
-                  if (res > 0) {
-                    if (o.debugging)
-                      log_write(LOG_STDOUT, "nxt line: %s", recvbuf);
-                    if (recvbuf[0] == '4' && recvbuf[1] == '2' && recvbuf[2] == '6') {
-                      target->ports.forgetPort(portarray[i], IPPROTO_TCP);
-                      if (o.debugging || o.verbose)
-                        log_write(LOG_STDOUT, "Changed my mind about port %i\n", portarray[i]);
-                    }
+                  if (o.debugging)
+                    log_write(LOG_STDOUT, "nxt line: %s", recvbuf);
+                  if (recvbuf[0] == '4' && recvbuf[1] == '2' && recvbuf[2] == '6') {
+                    target->ports.forgetPort(portarray[i], IPPROTO_TCP);
+                    if (o.debugging || o.verbose)
+                      log_write(LOG_STDOUT, "Changed my mind about port %i\n", portarray[i]);
                   }
                 }
               }
