@@ -13,7 +13,7 @@
 --
 -- A few notes about the safety of the engine, that is, the ability for
 -- a script developer to crash or otherwise stall NSE. The purpose of noting
--- these attack vectors is more to show the difficulty in accidently
+-- these attack vectors is more to show the difficulty in accidentally
 -- breaking the system than to indicate a user may wish to break the
 -- system through these means.
 --  - A script writer can use the undocumented Lua function newproxy
@@ -32,8 +32,8 @@
 
 local _VERSION = _VERSION;
 local MAJOR, MINOR = assert(_VERSION:match "^Lua (%d+).(%d+)$");
-if tonumber(MAJOR.."."..MINOR) < 5.2 then
-  error "NSE requires Lua 5.2 or newer. It looks like you're using an older version of nmap."
+if tonumber(MAJOR.."."..MINOR) < 5.3 then
+  error "NSE requires Lua 5.3 or newer. It looks like you're using an older version of nmap."
 end
 
 local NAME = "NSE";
@@ -51,6 +51,10 @@ local DESTRUCTOR = "NSE_DESTRUCTOR";
 local SELECTED_BY_NAME = "NSE_SELECTED_BY_NAME";
 local FORMAT_TABLE = "NSE_FORMAT_TABLE";
 local FORMAT_XML = "NSE_FORMAT_XML";
+local PARALLELISM = "NSE_PARALLELISM";
+
+-- Unique value indicating the action function is going to run.
+local ACTION_STARTING = {};
 
 -- This is a limit on the number of script instance threads running at once. It
 -- exists only to limit memory use when there are many open ports. It doesn't
@@ -115,21 +119,27 @@ local gsub = string.gsub;
 local lower = string.lower;
 local match = string.match;
 local sub = string.sub;
+local upper = string.upper;
 
 local table = require "table";
 local concat = table.concat;
 local insert = table.insert;
+local pack = table.pack;
 local remove = table.remove;
 local sort = table.sort;
 local unpack = table.unpack;
 
+local os = require "os"
+local time = os.time
+local difftime = os.difftime
+
 do -- Add loader to look in nselib/?.lua (nselib/ can be in multiple places)
   local function loader (lib)
-    lib = lib:gsub("%.", "/"); -- change Lua "module seperator" to directory separator
+    lib = lib:gsub("%.", "/"); -- change Lua "module separator" to directory separator
     local name = "nselib/"..lib..".lua";
     local type, path = cnse.fetchfile_absolute(name);
     if type == "file" then
-      return loadfile(path);
+      return assert(loadfile(path));
     else
       return "\n\tNSE failed to find "..name.." in search paths.";
     end
@@ -137,6 +147,20 @@ do -- Add loader to look in nselib/?.lua (nselib/ can be in multiple places)
   insert(package.searchers, 1, loader);
 end
 
+local lpeg = require "lpeg";
+local U = require "lpeg-utility"
+local locale = lpeg.locale;
+local P = lpeg.P;
+local R = lpeg.R;
+local S = lpeg.S;
+local V = lpeg.V;
+local C = lpeg.C;
+local Cb = lpeg.Cb;
+local Cc = lpeg.Cc;
+local Cf = lpeg.Cf;
+local Cg = lpeg.Cg;
+local Ct = lpeg.Ct;
+
 local nmap = require "nmap";
 local lfs = require "lfs";
 
@@ -162,7 +186,7 @@ local NSE_YIELD_VALUE = {};
 do
   -- This is the method by which we allow a script to have nested
   -- coroutines. If a sub-thread yields in an NSE function such as
-  -- nsock.connect, then we propogate the yield up. These replacements
+  -- nsock.connect, then we propagate the yield up. These replacements
   -- to the coroutine library are used only by Script Threads, not the engine.
 
   local function handle (co, status, ...)
@@ -215,6 +239,11 @@ local function log_error (fmt, ...)
   log_write("stderr", format(fmt, ...));
 end
 
+-- Check for and warn about some known bad behaviors
+if ("test"):gsub(".*$", "x") == "xx" then
+  log_error("Known bug in string.gsub in Lua 5.3 before 5.3.3 will cause bugs in NSE scripts.")
+end
+
 local function table_size (t)
   local n = 0; for _ in pairs(t) do n = n + 1; end return n;
 end
@@ -237,17 +266,7 @@ end
 
 -- recursively copy a table, for host/port tables
 -- not very rigorous, but it doesn't need to be
-local function tcopy (t)
-  local tc = {};
-  for k,v in pairs(t) do
-    if type(v) == "table" then
-      tc[k] = tcopy(v);
-    else
-      tc[k] = v;
-    end
-  end
-  return tc;
-end
+local tcopy = require "tableaux".tcopy
 
 -- copies the host table while preserving the registry
 local function host_copy(t)
@@ -256,6 +275,12 @@ local function host_copy(t)
   return h
 end
 
+-- Return a pattern which matches a "keyword" literal, case insensitive.
+local function K (a)
+  local insensitize = Cf((P(1) / function (a) return S(lower(a)..upper(a)) end)^1, function (a, b) return a * b end);
+  return assert(insensitize:match(a)) * #(V "space" + S"()," + P(-1)); -- "keyword" token
+end
+
 local REQUIRE_ERROR = {};
 rawset(stdnse, "silent_require", function (...)
   local status, mod = pcall(require, ...);
@@ -310,19 +335,28 @@ do
     return Worker[key] or self.parent[key]
   end
 
+  local function replace(fmt, pattern, repl)
+    -- Escape each % twice: once for gsub, and once for print_debug.
+    local r = gsub(repl, "%%", "%%%%%%%%")
+    return gsub(fmt, pattern, r);
+  end
   -- Thread:d()
   -- Outputs debug information at level 1 or higher.
   -- Changes "%THREAD" with an appropriate identifier for the debug level
   function Thread:d (fmt, ...)
     local against = against_name(self.host, self.port);
-    if debugging() > 1 then
-      fmt = gsub(fmt, "%%THREAD_AGAINST", self.info..against);
-      fmt = gsub(fmt, "%%THREAD", self.info);
+    local dbg = debugging()
+    if dbg > 1 then
+      fmt = replace(fmt, "%%THREAD_AGAINST", self.info..against);
+      fmt = replace(fmt, "%%THREAD", self.info);
+    elseif dbg == 1 then
+      fmt = replace(fmt, "%%THREAD_AGAINST", self.short_basename..against);
+      fmt = replace(fmt, "%%THREAD", self.short_basename);
     else
-      fmt = gsub(fmt, "%%THREAD_AGAINST", self.short_basename..against);
-      fmt = gsub(fmt, "%%THREAD", self.short_basename);
+      return
     end
-    print_debug(1, fmt, ...);
+    -- debugging() >= 1
+    log_write("stdout", format(fmt, ...));
   end
 
   -- Sets script output. r1 and r2 are the (as many as two) return values.
@@ -330,7 +364,7 @@ do
     if not self.worker then
       -- Structure table and unstructured string outputs.
       local tab, str
-  
+
       if r2 then
         tab, str = r1, tostring(r2);
       elseif type(r1) == "string" then
@@ -340,7 +374,7 @@ do
       else
         tab, str = r1, nil;
       end
-  
+
       if self.type == "prerule" or self.type == "postrule" then
         cnse.script_set_output(self.id, tab, str);
       elseif self.type == "hostrule" then
@@ -354,10 +388,16 @@ do
   -- prerule/postrule scripts may be timed out in the future
   -- based on start time and script lifetime?
   function Thread:timed_out ()
-    if self.type == "hostrule" or self.type == "portrule" then
-      return cnse.timedOut(self.host);
+    -- checking whether user gave --script-timeout option or not
+    if cnse.script_timeout and cnse.script_timeout > 0 and
+      -- comparing script's timeout with time elapsed
+      cnse.script_timeout < difftime(time(), self.start_time) then
+      return true
     end
-    return nil;
+    if self.host then
+      return cnse.timedOut(self.host)
+    end
+    return false
   end
 
   function Thread:start_time_out_clock ()
@@ -374,11 +414,16 @@ do
 
   -- Register scripts in the timeouts list to track their timeouts.
   function Thread:start (timeouts)
-    self:d("Starting %THREAD_AGAINST.");
     if self.host then
       timeouts[self.host] = timeouts[self.host] or {};
       timeouts[self.host][self.co] = true;
     end
+    -- storing script's start time so as to account for script's timeout later
+    if self.worker then
+      self.start_time = self.parent.start_time
+    else
+      self.start_time = time()
+    end
   end
 
   -- Remove scripts from the timeouts list and call their
@@ -410,7 +455,7 @@ do
   function Script:new_thread (rule, ...)
     local script_type = assert(NSE_SCRIPT_RULES[rule]);
     if not self[rule] then return nil end -- No rule for this script?
-    local script_closure_generator = self.script_closure_generator;
+
     -- Rebuild the environment for the running thread.
     local env = {
         SCRIPT_PATH = self.filename,
@@ -418,52 +463,50 @@ do
         SCRIPT_TYPE = script_type,
     };
     setmetatable(env, {__index = _G});
-    local script_closure = script_closure_generator(env);
-    local unique_value = {}; -- to test valid yield
-    local function main (_ENV, ...)
-      script_closure(); -- loads script globals
-      return action(yield(unique_value, _ENV[rule](...)));
+    local forced = self.forced_to_run;
+    local script_closure_generator = self.script_closure_generator;
+    local function main (...)
+      local _ENV = env; -- change the environment
+      -- Load the script's globals in the same Lua thread the action and rule
+      -- functions will execute in.
+      script_closure_generator(_ENV)();
+      if forced or _ENV[rule](...) then
+        yield(ACTION_STARTING)
+        return action(...)
+      end
     end
-    -- This thread allows us to load the script's globals in the
-    -- same Lua thread the action and rule functions will execute in.
+
     local co = create(main);
-    local s, value, rule_return = resume(co, env, ...);
-    if s and value ~= unique_value then
-      print_debug(1,
-    "A thread for %s yielded unexpectedly in the file or %s function:\n%s\n",
-          self.filename, rule, traceback(co));
-    elseif s and (rule_return or self.forced_to_run) then
-      local thread = {
-        close_handlers = {},
-        co = co,
-        env = env,
-        identifier = tostring(co),
-        info = format("'%s' (%s)", self.short_basename, tostring(co));
-        parent = nil, -- placeholder
-        script = self,
-        type = script_type,
-        worker = false,
-      };
-      thread.parent = thread;
-      setmetatable(thread, Thread)
-      return thread;
-    elseif not s then
-      log_error("A thread for %s failed to load in %s function:\n%s\n",
-          self.filename, rule, traceback(co, tostring(value)));
-    end
-    return nil;
+    local thread = {
+      action_started = false,
+      args = pack(...),
+      close_handlers = {},
+      co = co,
+      env = env,
+      identifier = tostring(co),
+      info = format("%s M:%s", self.id, match(tostring(co), "^thread: 0?[xX]?(.*)"));
+      parent = nil, -- placeholder
+      script = self,
+      type = script_type,
+      worker = false,
+      start_time = 0, --for script timeout
+    };
+    thread.parent = thread;
+    setmetatable(thread, Thread)
+    return thread;
   end
 
   function Thread:new_worker (main, ...)
     local co = create(main);
     print_debug(2, "%s spawning new thread (%s).", self.parent.info, tostring(co));
     local thread = {
-      args = {n = select("#", ...), ...},
+      args = pack(...),
       close_handlers = {},
       co = co,
-      info = format("'%s' worker (%s)", self.short_basename, tostring(co));
+      info = format("%s W:%s", self.id, match(tostring(co), "^thread: 0?[xX]?(.*)"));
       parent = self,
       worker = true,
+      start_time = 0,
     };
     setmetatable(thread, Worker)
     local function info ()
@@ -472,8 +515,47 @@ do
     return thread, info;
   end
 
-  function Thread:resume ()
-    return resume(self.co, unpack(self.args, 1, self.args.n));
+  function Thread:resume (timeouts)
+    local ok, r1, r2 = resume(self.co, unpack(self.args, 1, self.args.n));
+    local status = status(self.co);
+    if ok and r1 == ACTION_STARTING then
+      self:d("Starting %THREAD_AGAINST.");
+      self.action_started = true
+      return self:resume(timeouts);
+    elseif not ok then
+      -- Extend this to create new types of errors with custom handling.
+      -- nmap.new_try does equivalent of: error({errtype="nmap.new_try", message="TIMEOUT"})
+      if type(r1) == "table" and r1.errtype == "nmap.new_try" then
+        -- nmap.new_try "exception" is closing the script
+        if debugging() > 0 then
+          self:d("Finished %THREAD_AGAINST. Reason: %s\n", r1.message);
+        end
+        r1 = r1.message
+      elseif debugging() > 0 then
+        self:d("%THREAD_AGAINST threw an error!\n%s\n", traceback(self.co, tostring(r1)));
+      else
+        self:set_output("ERROR: Script execution failed (use -d to debug)");
+      end
+      self:close(timeouts, r1);
+      return false
+    elseif status == "suspended" then
+      if r1 == NSE_YIELD_VALUE then
+        return true
+      else
+        self:d("%THREAD yielded unexpectedly and cannot be resumed.");
+        self:close(timeouts, "yielded unexpectedly and cannot be resumed");
+        return false
+      end
+    elseif status == "dead" then
+      if self.action_started then
+        self:set_output(r1, r2);
+        -- -d1 = report finished scripts. -d2 = report finished threads
+        if not self.worker or debugging() > 1 then
+          self:d("Finished %THREAD_AGAINST.");
+        end
+      end
+      self:close(timeouts);
+    end
   end
 
   function Thread:__index (key)
@@ -572,12 +654,12 @@ do
     local postrule = rules.postrule;
     -- Assert that categories is an array of strings
     for i, category in ipairs(rawget(env, "categories")) do
-      assert(type(category) == "string", 
+      assert(type(category) == "string",
         filename.." has non-string entries in the 'categories' array");
     end
     -- Assert that dependencies is an array of strings
     for i, dependency in ipairs(rawget(env, "dependencies")) do
-      assert(type(dependency) == "string", 
+      assert(type(dependency) == "string",
         filename.." has non-string entries in the 'dependencies' array");
     end
     -- Return the script
@@ -624,7 +706,7 @@ end
 -- Arguments:
 --   rules  The array of rules to use for loading scripts.
 -- Returns:
---   chosen_scripts  The array of scripts loaded for the given rules. 
+--   chosen_scripts  The array of scripts loaded for the given rules.
 local function get_chosen_scripts (rules)
   check_rules(rules);
 
@@ -634,14 +716,7 @@ local function get_chosen_scripts (rules)
     "\tplease update using: nmap --script-updatedb");
 
   local chosen_scripts, files_loaded = {}, {};
-  local entry_rules, used_rules, forced_rules = {}, {}, {};
-
-  -- Tokens that are allowed in script rules (--script)
-  local protected_lua_tokens = {
-    ["and"] = true,
-    ["or"] = true,
-    ["not"] = true,
-  };
+  local used_rules, forced_rules = {}, {};
 
   -- Was this category selection forced to run (e.g. "+script").
   -- Return:
@@ -656,98 +731,75 @@ local function get_chosen_scripts (rules)
     end
   end
 
-  -- Globalize all names in str that are not protected_lua_tokens
-  local function globalize (str)
-    local lstr = lower(str);
-    if protected_lua_tokens[lstr] then
-      return lstr;
-    else
-      return 'm("'..str..'")';
-    end
-  end
-
   for i, rule in ipairs(rules) do
     rule = match(rule, "^%s*(.-)%s*$"); -- strip surrounding whitespace
-    local original_rule = rule;
     local forced, rule = is_forced_set(rule);
-    used_rules[rule] = false; -- has not been used yet
-    forced_rules[rule] = forced;
-    -- Here we escape backslashes which might appear in Windows filenames.
-    rule = gsub(rule, "\\([^\\])", "\\\\%1");
-    -- Globalize all `names`, all visible characters not ',', '(', ')', and ';'
-    local globalized_rule =
-        gsub(rule, "[\033-\039\042-\043\045-\058\060-\126]+", globalize);
-    -- Precompile the globalized rule
-    local env = {m = nil};
-    local compiled_rule, err = load("return "..globalized_rule, "rule", "t", env);
-    if not compiled_rule then
-      err = err:match("rule\"]:%d+:(.+)$"); -- remove (luaL_)where in code
-      error("Bad script rule:\n\t"..original_rule.." -> "..err);
+    if rule and rule ~= "" then
+      used_rules[rule] = false; -- has not been used yet
+      forced_rules[rule] = forced;
+      -- Here we escape backslashes which might appear in Windows filenames.
+      rule = gsub(rule, "\\([^\\])", "\\\\%1");
+      rules[i] = rule;
     end
-    -- These are used to reference and check all the rules later.
-    entry_rules[globalized_rule] = {
-      original_rule = rule,
-      compiled_rule = compiled_rule,
-      env = env,
-    };
   end
 
   -- Checks if a given script, script_entry, should be loaded. A script_entry
   -- should be in the form: { filename = "name.nse", categories = { ... } }
   function db_env.Entry (script_entry)
-    local categories, filename = script_entry.categories, script_entry.filename;
-    assert(type(categories) == "table" and type(filename) == "string",
-        "script database appears corrupt, try `nmap --script-updatedb`");
-    local escaped_basename = match(filename, "([^/\\]-)%.nse$") or
-                             match(filename, "([^/\\]-)$");
-
-    local r_categories = {all = true}; -- A reverse table of categories
-    for i, category in ipairs(categories) do
-      assert(type(category) == "string", "bad entry in script database");
-      r_categories[lower(category)] = true; -- Lowercase the entry
-    end
-
+    local categories = rawget(script_entry, "categories");
+    local filename = rawget(script_entry, "filename");
+    assert(type(categories) == "table" and type(filename) == "string", "script database appears corrupt, try `nmap --script-updatedb`");
+    local escaped_basename = match(filename, "([^/\\]-)%.nse$") or match(filename, "([^/\\]-)$");
+    local selected_by_name = false;
     -- The script selection parameters table.
     local script_params = {};
 
-    -- A matching function for each script rule.
-    -- If the pattern directly matches a category (e.g. "all"), then
-    -- we return true. Otherwise we test if it is a filename or if
-    -- the script_entry.filename matches the pattern.
-    local function m (pattern)
-      -- Check categories
-      if r_categories[lower(pattern)] then
-        script_params.selection = "category";
-        return true;
-      end
-
-      -- Check filename with wildcards
-      pattern = gsub(pattern, "%.nse$", ""); -- remove optional extension
-      pattern = gsub(pattern, "[%^%$%(%)%%%.%[%]%+%-%?]", "%%%1"); -- esc magic
-      pattern = gsub(pattern, "%*", ".*"); -- change to Lua wildcard
-      pattern = "^"..pattern.."$"; -- anchor to beginning and end
-      if find(escaped_basename, pattern) then
-        script_params.selection = "name";
-        script_params.verbosity = true;
-        return true;
-      end
-
-      return false;
+    -- Test if path is a glob pattern that matches script_entry.filename.
+    local function match_script (path)
+      path = gsub(path, "%.nse$", ""); -- remove optional extension
+      path = gsub(path, "[%^%$%(%)%%%.%[%]%+%-%?]", "%%%1"); -- esc magic
+      path = gsub(path, "%*", ".*"); -- change to Lua wildcard
+      path = "^"..path.."$"; -- anchor to beginning and end
+      local found = not not find(escaped_basename, path);
+      selected_by_name = selected_by_name or found;
+      return found;
     end
 
-    for globalized_rule, rule_table in pairs(entry_rules) do
-      -- Clear and set the environment of the compiled script rule
-      rule_table.env.m = m;
-      local status, found = pcall(rule_table.compiled_rule)
-      rule_table.env.m = nil;
-      if not status then
-        error("Bad script rule:\n\t"..rule_table.original_rule..
-              " -> script rule expression not supported.");
-      end
-      -- The script rule matches a category or a pattern
-      if found then 
-        used_rules[rule_table.original_rule] = true;
-        script_params.forced = not not forced_rules[rule_table.original_rule];
+    local T = locale {
+      V "space"^0 * V "expression" * V "space"^0 * P(-1);
+
+      expression = V "disjunct" + V "conjunct" + V "value";
+      disjunct = (V "conjunct" + V "value") * V "space"^0 * K "or" * V "space"^0 * V "expression" / function (a, b) return a or b end;
+      conjunct = V "value" * V "space"^0 * K "and" * V "space"^0 * V "expression" / function (a, b) return a and b end;
+      value = K "not" * V "space"^0 * V "value" / function (a) return not a end +
+              P "(" * V "space"^0 * V "expression" * V "space"^0 * P ")" +
+              K "true" * Cc(true) +
+              K "false" * Cc(false) +
+              V "category" +
+              V "path";
+
+      category = K "all" * Cc(true); -- pseudo-category "all" matches everything
+      path = R("\033\039", "\042\126")^1 / match_script; -- all graphical characters not '(', ')'
+    };
+
+    for i, category in ipairs(categories) do
+      assert(type(category) == "string", "bad entry in script database");
+      T.category = T.category + K(category) * Cc(true);
+    end
+
+    T = P(T);
+
+    for i, rule in ipairs(rules) do
+      selected_by_name = false;
+      if T:match(rule) then
+        used_rules[rule] = true;
+        script_params.forced = not not forced_rules[rule];
+        if selected_by_name then
+          script_params.selection = "name"
+          script_params.verbosity = true
+        else
+          script_params.selection = "category"
+        end
         local t, path = cnse.fetchscript(filename);
         if t == "file" then
           if not files_loaded[path] then
@@ -776,7 +828,12 @@ local function get_chosen_scripts (rules)
         t, path = cnse.fetchscript(rule..".nse");
       end
       if t == nil then
-        error("'"..rule.."' did not match a category, filename, or directory");
+        -- Avoid erroring if -sV but no scripts are present
+        if not (cnse.scriptversion and rule == "version") then
+          error("'"..rule.."' did not match a category, filename, or directory");
+        end
+      elseif t == "bare_directory" then
+        error("directory '"..path.."' found, but will not match without '/'")
       elseif t == "file" and not files_loaded[path] then
         script_params.selection = "file path";
         script_params.verbosity = true;
@@ -800,13 +857,14 @@ local function get_chosen_scripts (rules)
   -- calculate runlevels
   local name_script = {};
   for i, script in ipairs(chosen_scripts) do
-    assert(name_script[script.short_basename] == nil);
+    assert(name_script[script.short_basename] == nil,
+      ("duplicate script ID: '%s'"):format(script.short_basename));
     name_script[script.short_basename] = script;
   end
   local chain = {}; -- chain of script names
   local function calculate_runlevel (script)
     chain[#chain+1] = script.short_basename;
-    if script.runlevel == false then -- circular dependency 
+    if script.runlevel == false then -- circular dependency
       error("circular dependency in chain `"..concat(chain, "->").."`");
     else
       script.runlevel = false; -- placeholder
@@ -839,7 +897,8 @@ local function run (threads_iter, hosts)
   -- running scripts may be resumed at any time. waiting scripts are
   -- yielded until Nsock wakes them. After being awakened with
   -- nse_restore, waiting threads become pending and later are moved all
-  -- at once back to running.
+  -- at once back to running. pending is used because we cannot modify
+  -- running during traversal.
   local running, waiting, pending = {}, {}, {};
   local all = setmetatable({}, {__mode = "kv"}); -- base coroutine to Thread
   local current; -- The currently running Thread.
@@ -864,7 +923,7 @@ local function run (threads_iter, hosts)
       co = base.co;
       if waiting[co] then -- ignore a thread not waiting
         pending[co], waiting[co] = waiting[co], nil;
-        pending[co].args = {n = select("#", ...), ...};
+        pending[co].args = pack(...);
       end
     end
   end
@@ -900,20 +959,23 @@ local function run (threads_iter, hosts)
   rawset(stdnse, "base", function ()
     return current and current.co;
   end);
-
-  while threads_iter and num_threads < CONCURRENCY_LIMIT do
-    local thread = threads_iter()
-    if not thread then
-      threads_iter = nil;
-      break;
+  rawset(stdnse, "gettid", function ()
+    return current and current.identifier;
+  end);
+  rawset(stdnse, "getid", function ()
+    return current and current.id;
+  end);
+  rawset(stdnse, "getinfo", function ()
+    return current and current.info;
+  end);
+  rawset(stdnse, "gethostport", function ()
+    if current then
+        return current.host, current.port;
     end
-    all[thread.co], running[thread.co], total = thread, thread, total+1;
-    num_threads = num_threads + 1;
-    thread:start(timeouts);
-  end
-  if num_threads == 0 then
-    return
-  end
+  end);
+  rawset(stdnse, "isworker", function ()
+    return current and current.worker;
+  end);
 
   local progress = cnse.scan_progress_meter(NAME);
 
@@ -932,21 +994,57 @@ local function run (threads_iter, hosts)
     end
 
     local nr, nw = table_size(running), table_size(waiting);
-    if cnse.key_was_pressed() then
-      print_verbose(1, "Active NSE Script Threads: %d (%d waiting)\n",
+    -- total may be 0 if no scripts are running in this phase
+    if total > 0 and cnse.key_was_pressed() then
+      print_verbose(1, "Active NSE Script Threads: %d (%d waiting)",
           nr+nw, nw);
       progress("printStats", 1-(nr+nw)/total);
       if debugging() >= 2 then
         for co, thread in pairs(running) do
-          thread:d("Running: %THREAD\n\t%s",
+          thread:d("Running: %THREAD_AGAINST\n\t%s",
               (gsub(traceback(co), "\n", "\n\t")));
         end
         for co, thread in pairs(waiting) do
-          thread:d("Waiting: %THREAD\n\t%s",
+          thread:d("Waiting: %THREAD_AGAINST\n\t%s",
               (gsub(traceback(co), "\n", "\n\t")));
         end
+      elseif debugging() >= 1 then
+        local display = {}
+        local limit = 0
+        for co, thread in pairs(running) do
+          local this = display[thread.short_basename]
+          if not this then
+            this = {}
+            limit = limit + 1
+            if limit > 5 then
+              -- Only print stats if 5 or fewer scripts remaining
+              break
+            end
+          end
+          this[1] = (this[1] or 0) + 1
+          display[thread.short_basename] = this
+        end
+        for co, thread in pairs(waiting) do
+          local this = display[thread.short_basename]
+          if not this then
+            this = {}
+            limit = limit + 1
+            if limit > 5 then
+              -- Only print stats if 5 or fewer scripts remaining
+              break
+            end
+          end
+          this[2] = (this[2] or 0) + 1
+          display[thread.short_basename] = this
+        end
+        if limit <= 5 then
+          for name, stats in pairs(display) do
+            print_debug(1, "Script %s: %d threads running, %d threads waiting",
+              name, stats[1] or 0, stats[2] or 0)
+          end
+        end
       end
-    elseif progress "mayBePrinted" then
+    elseif total > 0 and progress "mayBePrinted" then
       if verbosity() > 1 or debugging() > 0 then
         progress("printStats", 1-(nr+nw)/total);
       else
@@ -954,15 +1052,15 @@ local function run (threads_iter, hosts)
       end
     end
 
+    local orphans = true
     -- Checked for timed-out scripts and hosts.
     for co, thread in pairs(waiting) do
       if thread:timed_out() then
         waiting[co], all[co], num_threads = nil, nil, num_threads-1;
-        thread:d("%THREAD %stimed out", thread.host
-            and format("%s%s ", thread.host.ip,
-                    thread.port and ":"..thread.port.number or "")
-            or "");
+        thread:d("%THREAD_AGAINST timed out")
         thread:close(timeouts, "timed out");
+      elseif not thread.worker then
+        orphans = false
       end
     end
 
@@ -970,29 +1068,13 @@ local function run (threads_iter, hosts)
       current, running[co] = thread, nil;
       thread:start_time_out_clock();
 
-      -- Threads may have zero, one, or two return values.
-      local s, r1, r2 = thread:resume();
-      if not s then -- script error...
-        all[co], num_threads = nil, num_threads-1;
-        if debugging() > 0 then
-          thread:d("%THREAD_AGAINST threw an error!\n%s\n", traceback(co, tostring(r1)));
-        else
-          thread:set_output("ERROR: Script execution failed (use -d to debug)");
+      if thread:resume(timeouts) then
+        waiting[co] = thread;
+        if not thread.worker then
+          orphans = false
         end
-        thread:close(timeouts, r1);
-      elseif status(co) == "suspended" then
-        if r1 == NSE_YIELD_VALUE then
-          waiting[co] = thread;
-        else
-          all[co], num_threads = nil, num_threads-1;
-          thread:d("%THREAD yielded unexpectedly and cannot be resumed.");
-          thread:close();
-        end
-      elseif status(co) == "dead" then
+      else
         all[co], num_threads = nil, num_threads-1;
-        thread:set_output(r1, r2);
-        thread:d("Finished %THREAD_AGAINST.");
-        thread:close(timeouts);
       end
       current = nil;
     end
@@ -1001,9 +1083,17 @@ local function run (threads_iter, hosts)
     -- Move pending threads back to running.
     for co, thread in pairs(pending) do
       pending[co], running[co] = nil, thread;
+      if not thread.worker then
+        orphans = false
+      end
     end
 
     collectgarbage "step";
+    -- If we didn't see at least one non-worker thread, then any remaining are orphaned.
+    if num_threads > 0 and orphans then
+      print_debug(1, "%d orphans left!", total)
+      break
+    end
   end
 
   progress "endTask";
@@ -1028,15 +1118,21 @@ local function format_table(obj, indent)
     local lines = {};
     -- Do integer keys.
     for _, v in ipairs(obj) do
-      lines[#lines + 1] = indent .. format_table(v, indent .. "  ");
+      lines[#lines + 1] = "\n"
+      lines[#lines + 1] = indent
+      lines[#lines + 1] = format_table(v, indent .. "  ")
     end
     -- Do string keys.
     for k, v in pairs(obj) do
       if type(k) == "string" then
-        lines[#lines + 1] = indent .. k .. ": " .. format_table(v, indent .. "  ");
+        lines[#lines + 1] = "\n"
+        lines[#lines + 1] = indent
+        lines[#lines + 1] = k
+        lines[#lines + 1] = ": "
+        lines[#lines + 1] = format_table(v, indent .. "  ")
       end
     end
-    return "\n" .. concat(lines, "\n");
+    return concat(lines);
   else
     return tostring(obj);
   end
@@ -1150,85 +1246,62 @@ local function script_help_xml(chosen_scripts)
   cnse.xml_newline();
 end
 
-do -- Load script arguments (--script-args)
-  local args = cnse.scriptargs or "";
-  print_debug(1, "Script Arguments seen from CLI: %s", args);
+nmap.registry.args = {};
+do
+  local args = {};
 
-  -- Parse a string in 'str' at 'start'.
-  local function parse_string (str, start)
-    -- Unquoted
-    local uqi, uqj, uqm = find(str,
-        "^%s*([^'\"%s{},=][^{},=]-)%s*[},=]", start);
-    -- Quoted
-    local qi, qj, q, qm = find(str, "^%s*(['\"])(.-[^\\])%1%s*[},=]", start);
-    -- Empty Quote
-    local eqi, eqj = find(str, "^%s*(['\"])%1%s*[},=]", start);
-    if uqi then
-      return uqm, uqj-1;
-    elseif qi then
-      return gsub(qm, "\\"..q, q), qj-1;
-    elseif eqi then
-      return "", eqj-1;
-    else
-      error("Value around '"..sub(str, start, start+10)..
-          "' is invalid or is unterminated by a valid seperator");
-    end
-  end
-  -- Takes 'str' at index 'start' and parses a table. 
-  -- Returns the table and the place in the string it finished reading.
-  local function parse_table (str, start)
-    local _, j = find(str, "^%s*{", start);
-    local t = {}; -- table we return
-    local tmp, nc; -- temporary and next character inspected
-
-    while true do
-      j = j+1; -- move past last token
-
-      _, j, nc = find(str, "^%s*(%S)", j);
-
-      if nc == "}" then -- end of table
-        return t, j;
-      else -- try to read key/value pair, or array value
-        local av = false; -- this is an array value?
-        if nc == "{" then -- array value
-          av, tmp, j = true, parse_table(str, j);
-        else
-          tmp, j = parse_string(str, j);
-        end
-        nc = sub(str, j+1, j+1); -- next token
-        if not av and nc == "=" then -- key/value?
-          _, j, nc = find(str, "^%s*(%S)", j+2);
-          if nc == "{" then
-            t[tmp], j = parse_table(str, j);
-          else -- regular string
-            t[tmp], j = parse_string(str, j);
-          end
-          nc = sub(str, j+1, j+1); -- next token
-        else -- not key/value pair, save array value
-          t[#t+1] = tmp;
-        end
-        if nc == "," then j = j+1 end -- skip "," token
-      end
-    end
-  end
-  nmap.registry.args = parse_table("{"..args.."}", 1);
-  -- Check if user wants to read scriptargs from a file
-  if cnse.scriptargsfile ~= nil then --scriptargsfile path/to/file
+  if cnse.scriptargsfile then
     local t, path = cnse.fetchfile_absolute(cnse.scriptargsfile)
     assert(t == 'file', format("%s is not a file", path))
-    local argfile = assert(open(path, 'r'));
-    local argstring = argfile:read("*a")
-    argstring = gsub(argstring,"\n",",")
-    local tmpargs = parse_table("{"..argstring.."}",1)
-    for k,v in pairs(nmap.registry.args) do
-      tmpargs[k] = v
-    end
-    nmap.registry.args = tmpargs
+    print_debug(1, "Loading script-args from file `%s'", cnse.scriptargsfile);
+    args[#args+1] = assert(assert(open(path, 'r')):read "*a"):gsub(",*$", "");
   end
-  if debugging() >= 2 then
-    local out = {}
-    rawget(stdnse, "pretty_printer")(nmap.registry.args, function (s) out[#out+1] = s end)
-    print_debug(2, concat(out))
+
+  if cnse.scriptargs then -- Load script arguments (--script-args)
+    print_debug(1, "Arguments from CLI: %s", cnse.scriptargs);
+    args[#args+1] = cnse.scriptargs;
+  end
+
+  if cnse.script_timeout and cnse.script_timeout > 0 then
+    print_debug(1, "Set script-timeout as: %d seconds", cnse.script_timeout);
+  end
+
+  args = concat(args, ",");
+  if #args > 0 then
+    print_debug(1, "Arguments parsed: %s", args);
+    local function set (t, a, b)
+      if b == nil then
+        insert(t, a);
+        return t;
+       else
+        return rawset(t, a, b);
+      end
+    end
+    local parser = locale {
+      V "space"^0 * V "table" * V "space"^0,
+      table = Cf(Ct "" * P "{" * V "space"^0 * (V "fieldlst")^-1 * V "space"^0 * P "}", set);
+      hws = V "space" - P "\n", -- horizontal whitespace
+      fieldlst = V "field" * (V "hws"^0 * S "\n," * V "space"^0 * V "field")^0;
+      field = V "kv" + V "av";
+      kv = Cg(V "string" * V "hws"^0 * P "=" * V "hws"^0 * V "value");
+      av = Cg(V "value");
+      value = V "table" + V "string";
+      string = V "qstring" + V "uqstring";
+      qstring = U.escaped_quote('"') + U.escaped_quote("'");
+      uqstring = V "hws"^0 * C((P(1) - V "hws"^0 * S "\n,{}=")^0) * V "hws"^0; -- everything but '\n,{}=', do not capture final space
+    };
+    --U.debug(parser,function(...)return print_debug(1,...)end)
+    parser = assert(P(parser));
+    nmap.registry.args = parser:match("{"..args.."}");
+    if not nmap.registry.args then
+      log_write("stdout", "args = "..args);
+      error "arguments did not parse!"
+    end
+    if debugging() >= 2 then
+      local out = {}
+      rawget(stdnse, "pretty_printer")(nmap.registry.args, function (s) out[#out+1] = s end)
+      print_debug(2, "%s", concat(out))
+    end
   end
 end
 
@@ -1303,13 +1376,13 @@ local function main (hosts, scantype)
   --  parent  A table that contains the parent thread table (it self).
   --  close_handlers
   --          A table that contains the thread destructor handlers.
-  --  info    A string that contains the script name and the thread 
+  --  info    A string that contains the script name and the thread
   --            debug information.
-  --  args    A table that contains the arguments passed to scripts, 
+  --  args    A table that contains the arguments passed to scripts,
   --            arguments can be host and port tables.
   --  env     A table that contains the global script environment:
   --            categories, description, author, license, nmap table,
-  --            action function, rule functions, SCRIPT_PATH, 
+  --            action function, rule functions, SCRIPT_PATH,
   --            SCRIPT_NAME, SCRIPT_TYPE (pre|host|port|post rule).
   --  identifier
   --          A string to identify the thread address.
@@ -1324,6 +1397,10 @@ local function main (hosts, scantype)
     insert(runlevels[script.runlevel], script);
   end
 
+  if _R[PARALLELISM] > CONCURRENCY_LIMIT then
+    CONCURRENCY_LIMIT = _R[PARALLELISM];
+  end
+
   if scantype == NSE_PRE_SCAN then
     print_verbose(1, "Script Pre-scanning.");
   elseif scantype == NSE_SCAN then
@@ -1336,10 +1413,6 @@ local function main (hosts, scantype)
     print_verbose(1, "Script Post-scanning.");
   end
 
-  -- These functions do not exist until we are executing action functions.
-  rawset(stdnse, "new_thread", nil)
-  rawset(stdnse, "base", nil)
-
   for runlevel, scripts in ipairs(runlevels) do
     -- This iterator is passed to the run function. It returns one new script
     -- thread on demand until exhausted.
@@ -1349,8 +1422,7 @@ local function main (hosts, scantype)
         for _, script in ipairs(scripts) do
            local thread = script:new_thread("prerule");
            if thread then
-             thread.args = {n = 0};
-             yield(thread);
+             yield(thread)
            end
         end
       -- activate hostrule and portrule scripts
@@ -1360,7 +1432,7 @@ local function main (hosts, scantype)
           for _, script in ipairs(scripts) do
             local thread = script:new_thread("hostrule", host_copy(host));
             if thread then
-              thread.args, thread.host = {n = 1, host_copy(host)}, host;
+              thread.host = host;
               yield(thread);
             end
           end
@@ -1369,7 +1441,7 @@ local function main (hosts, scantype)
             for _, script in ipairs(scripts) do
               local thread = script:new_thread("portrule", host_copy(host), tcopy(port));
               if thread then
-                thread.args, thread.host, thread.port = {n = 2, host_copy(host), tcopy(port)}, host, port;
+                thread.host, thread.port = host, port;
                 yield(thread);
               end
             end
@@ -1380,7 +1452,6 @@ local function main (hosts, scantype)
         for _, script in ipairs(scripts) do
           local thread = script:new_thread("postrule");
           if thread then
-            thread.args = {n = 0};
             yield(thread);
           end
         end
