@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2020 Insecure.Com LLC ("The Nmap  *
+ * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
  * Project"). Nmap is also a registered trademark of the Nmap Project.     *
  *                                                                         *
  * This program is distributed under the terms of the Nmap Public Source   *
@@ -13,9 +13,9 @@
  * file distributed with that version of Nmap or source code control       *
  * revision. More Nmap copyright/legal information is available from       *
  * https://nmap.org/book/man-legal.html, and further information on the    *
- * NPSL license itself can be found at https://nmap.org/npsl. This header  *
- * summarizes some key points from the Nmap license, but is no substitute  *
- * for the actual license text.                                            *
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+ * header summarizes some key points from the Nmap license, but is no      *
+ * substitute for the actual license text.                                 *
  *                                                                         *
  * Nmap is generally free for end users to download and use themselves,    *
  * including commercial use. It is available from https://nmap.org.        *
@@ -23,14 +23,14 @@
  * The Nmap license generally prohibits companies from using and           *
  * redistributing Nmap in commercial products, but we sell a special Nmap  *
  * OEM Edition with a more permissive license and special features for     *
- * this purpose. See https://nmap.org/oem                                  *
+ * this purpose. See https://nmap.org/oem/                                 *
  *                                                                         *
  * If you have received a written Nmap license agreement or contract       *
  * stating terms other than these (such as an Nmap OEM license), you may   *
  * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
  * The official Nmap Windows builds include the Npcap software             *
- * (https://npcap.org) for packet capture and transmission. It is under    *
+ * (https://npcap.com) for packet capture and transmission. It is under    *
  * separate license terms which forbid redistribution without special      *
  * permission. So the official Nmap Windows builds may not be              *
  * redistributed without special permission (such as an Nmap OEM           *
@@ -55,11 +55,11 @@
  * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
  * indemnification and commercial support are all available through the    *
- * Npcap OEM program--see https://nmap.org/oem.                            *
+ * Npcap OEM program--see https://nmap.org/oem/                            *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: portlist.h 38078 2020-10-02 16:12:22Z dmiller $ */
+/* $Id: portlist.h 38353 2022-02-18 17:38:46Z dmiller $ */
 
 #ifndef PORTLIST_H
 #define PORTLIST_H
@@ -203,7 +203,7 @@ class PortList {
    except that if you ask for TCP, UDP & SCTP, all TCP ports will be
    returned before we start returning UDP and finally SCTP ports */
   Port *nextPort(const Port *cur, Port *next,
-                 int allowed_protocol, int allowed_state);
+                 int allowed_protocol, int allowed_state) const;
 
   int setStateReason(u16 portno, u8 proto, reason_t reason, u8 ttl, const struct sockaddr_storage *ip_addr);
 
@@ -240,7 +240,7 @@ class PortList {
   void getServiceDeductions(u16 portno, int protocol, struct serviceDeductions *sd) const;
 
 #ifndef NOLUA
-  void addScriptResult(u16 portno, int protocol, const ScriptResult& sr);
+  void addScriptResult(u16 portno, int protocol, ScriptResult *sr);
 #endif
 
   /* Cycles through the 0 or more "ignored" ports which should be
@@ -251,13 +251,13 @@ class PortList {
    most popular one.  Returns the state if there is one, but returns
    PORT_UNKNOWN if there are no (more) states which qualify for
    consolidation */
-  int nextIgnoredState(int prevstate);
+  int nextIgnoredState(int prevstate) const;
 
   /* Returns true if a state should be ignored (consolidated), false otherwise */
-  bool isIgnoredState(int state);
+  bool isIgnoredState(int state, int *count) const;
 
-  int numIgnoredStates();
-  int numIgnoredPorts();
+  int numIgnoredStates() const;
+  int numIgnoredPorts() const;
   int numPorts() const;
   bool hasOpenPorts() const;
 
