@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2020 Insecure.Com LLC ("The Nmap  *
+ * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
  * Project"). Nmap is also a registered trademark of the Nmap Project.     *
  *                                                                         *
  * This program is distributed under the terms of the Nmap Public Source   *
@@ -14,9 +14,9 @@
  * file distributed with that version of Nmap or source code control       *
  * revision. More Nmap copyright/legal information is available from       *
  * https://nmap.org/book/man-legal.html, and further information on the    *
- * NPSL license itself can be found at https://nmap.org/npsl. This header  *
- * summarizes some key points from the Nmap license, but is no substitute  *
- * for the actual license text.                                            *
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+ * header summarizes some key points from the Nmap license, but is no      *
+ * substitute for the actual license text.                                 *
  *                                                                         *
  * Nmap is generally free for end users to download and use themselves,    *
  * including commercial use. It is available from https://nmap.org.        *
@@ -24,14 +24,14 @@
  * The Nmap license generally prohibits companies from using and           *
  * redistributing Nmap in commercial products, but we sell a special Nmap  *
  * OEM Edition with a more permissive license and special features for     *
- * this purpose. See https://nmap.org/oem                                  *
+ * this purpose. See https://nmap.org/oem/                                 *
  *                                                                         *
  * If you have received a written Nmap license agreement or contract       *
  * stating terms other than these (such as an Nmap OEM license), you may   *
  * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
  * The official Nmap Windows builds include the Npcap software             *
- * (https://npcap.org) for packet capture and transmission. It is under    *
+ * (https://npcap.com) for packet capture and transmission. It is under    *
  * separate license terms which forbid redistribution without special      *
  * permission. So the official Nmap Windows builds may not be              *
  * redistributed without special permission (such as an Nmap OEM           *
@@ -56,11 +56,11 @@
  * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
  * indemnification and commercial support are all available through the    *
- * Npcap OEM program--see https://nmap.org/oem.                            *
+ * Npcap OEM program--see https://nmap.org/oem/                            *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: services.cc 38078 2020-10-02 16:12:22Z dmiller $ */
+/* $Id: services.cc 38353 2022-02-18 17:38:46Z dmiller $ */
 
 #include "scan_lists.h"
 #include "services.h"
@@ -115,15 +115,15 @@ static int nmap_services_init() {
 
   char filename[512];
   FILE *fp;
-  char servicename[128], proto[16];
+  char servicename[128], proto[16] = { 0 };
   u16 portno;
-  char *p;
+  const char *p;
   char line[1024];
   int lineno = 0;
   int res;
   double ratio;
   int ratio_n, ratio_d;
-  char ratio_str[32];
+  char ratio_str[32] = { 0 };
 
   numtcpports = 0;
   numudpports = 0;
@@ -133,32 +133,21 @@ static int nmap_services_init() {
   ratio_format = 0;
 
   if (nmap_fetchfile(filename, sizeof(filename), "nmap-services") != 1) {
-#ifndef WIN32
     error("Unable to find nmap-services!  Resorting to /etc/services");
+#ifndef WIN32
     strcpy(filename, "/etc/services");
 #else
-        int len, wnt = GetVersion() < 0x80000000;
-    error("Unable to find nmap-services!  Resorting to /etc/services");
-        if(wnt)
-                len = GetSystemDirectory(filename, 480);	//	be safe
-        else
-                len = GetWindowsDirectory(filename, 480);	//	be safe
-        if(!len)
-                error("Get%sDirectory failed (%d) @#!#@",
-                 wnt ? "System" : "Windows", GetLastError());
-        else
-        {
-                if(wnt)
-                        strcpy(filename + len, "\\drivers\\etc\\services");
-                else
-                        strcpy(filename + len, "\\services");
-        }
+    int len = GetSystemDirectory(filename, 480);	//	be safe
+    if(!len)
+      fatal("GetSystemDirectory failed (%d) @#!#@", GetLastError());
+    else
+      strcpy(filename + len, "\\drivers\\etc\\services");
 #endif
   }
 
   fp = fopen(filename, "r");
   if (!fp) {
-    fatal("Unable to open %s for reading service information", filename);
+    pfatal("Unable to open %s for reading service information", filename);
   }
   /* Record where this data file was found. */
   o.loaded_data_files["nmap-services"] = filename;
@@ -208,7 +197,7 @@ static int nmap_services_init() {
     ps.proto = proto;
 
     /* Now we make sure our service table doesn't have duplicates */
-    std::map<port_spec, service_node>::iterator i;
+    std::map<port_spec, service_node>::const_iterator i;
     i = service_table.find(ps);
     if (i != service_table.end()) {
       if (o.debugging)
@@ -270,15 +259,15 @@ void free_services() {
  * Returns the number of ports added in total.
  */
 
-int addportsfromservmask(char *mask, u8 *porttbl, int range_type) {
-  std::map<port_spec, service_node>::iterator i;
+int addportsfromservmask(const char *mask, u8 *porttbl, int range_type) {
+  std::map<port_spec, service_node>::const_iterator i;
   int t = 0;
 
   if (!services_initialized && nmap_services_init() == -1)
     fatal("%s: Couldn't get port numbers", __func__);
 
   for (i = service_table.begin(); i != service_table.end(); i++) {
-    service_node& current = i->second;
+    const service_node& current = i->second;
     if (wildtest(mask, current.s_name)) {
       if ((range_type & SCAN_TCP_PORT) && strcmp(current.s_proto, "tcp") == 0) {
         porttbl[current.s_port] |= SCAN_TCP_PORT;
@@ -300,8 +289,8 @@ int addportsfromservmask(char *mask, u8 *porttbl, int range_type) {
 
 
 
-struct servent *nmap_getservbyport(int port, const char *proto) {
-  std::map<port_spec, service_node>::iterator i;
+const struct servent *nmap_getservbyport(int port, const char *proto) {
+  std::map<port_spec, service_node>::const_iterator i;
   port_spec ps;
 
   if (nmap_services_init() == -1)
@@ -368,12 +357,12 @@ static bool is_port_member(const struct scan_lists *ptsdata, const struct servic
 // This function doesn't support IP protocol scan so only call this
 // function if o.TCPScan() || o.UDPScan() || o.SCTPScan()
 
-void gettoppts(double level, char *portlist, struct scan_lists * ports, char *exclude_ports) {
+void gettoppts(double level, const char *portlist, struct scan_lists * ports, const char *exclude_ports) {
   int ti=0, ui=0, si=0;
   struct scan_lists ptsdata = { 0 };
   bool ptsdata_initialized = false;
   const struct service_node *current;
-  std::list<service_node>::iterator i;
+  std::list<service_node>::const_iterator i;
 
   if (!services_initialized && nmap_services_init() == -1)
     fatal("%s: Couldn't get port numbers", __func__);
