@@ -7,99 +7,72 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
- * also a registered trademark of Insecure.Com LLC.  This program is free  *
- * software; you may redistribute and/or modify it under the terms of the  *
- * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
+ * Project"). Nmap is also a registered trademark of the Nmap Project.     *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * This program is distributed under the terms of the Nmap Public Source   *
+ * License (NPSL). The exact license text applying to a particular Nmap    *
+ * release or source code control revision is contained in the LICENSE     *
+ * file distributed with that version of Nmap or source code control       *
+ * revision. More Nmap copyright/legal information is available from       *
+ * https://nmap.org/book/man-legal.html, and further information on the    *
+ * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+ * header summarizes some key points from the Nmap license, but is no      *
+ * substitute for the actual license text.                                 *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * Nmap is generally free for end users to download and use themselves,    *
+ * including commercial use. It is available from https://nmap.org.        *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@insecure.com for     *
- * further information.                                                    *
+ * The Nmap license generally prohibits companies from using and           *
+ * redistributing Nmap in commercial products, but we sell a special Nmap  *
+ * OEM Edition with a more permissive license and special features for     *
+ * this purpose. See https://nmap.org/oem/                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * If you have received a written Nmap license agreement or contract       *
+ * stating terms other than these (such as an Nmap OEM license), you may   *
+ * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * The official Nmap Windows builds include the Npcap software             *
+ * (https://npcap.com) for packet capture and transmission. It is under    *
+ * separate license terms which forbid redistribution without special      *
+ * permission. So the official Nmap Windows builds may not be              *
+ * redistributed without special permission (such as an Nmap OEM           *
+ * license).                                                               *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
- * This also allows you to audit the software for security holes (none     *
- * have been found so far).                                                *
+ * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify otherwise) *
- * that you are offering the Nmap Project (Insecure.Com LLC) the           *
- * unlimited, non-exclusive right to reuse, modify, and relicense the      *
- * code.  Nmap will always be available Open Source, but this is important *
- * because the inability to relicense code has caused devastating problems *
- * for other Free Software projects (such as KDE and NASM).  We also       *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
+ * and add new features.  You are highly encouraged to submit your         *
+ * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+ * for possible incorporation into the main distribution. Unless you       *
+ * specify otherwise, it is understood that you are offering us very       *
+ * broad rights to use your submissions as described in the Nmap Public    *
+ * Source License Contributor Agreement. This is important because we      *
+ * fund the project by selling licenses with various terms, and also       *
+ * because the inability to relicense code has caused devastating          *
+ * problems for other Free Software projects (such as KDE and NASM).       *
  *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * The free version of Nmap is distributed in the hope that it will be     *
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+ * indemnification and commercial support are all available through the    *
+ * Npcap OEM program--see https://nmap.org/oem/                            *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: timing.cc 28415 2012-04-07 08:16:08Z david $ */
+/* $Id: timing.cc 38353 2022-02-18 17:38:46Z dmiller $ */
 
 #include "timing.h"
 #include "NmapOps.h"
 #include "utils.h"
+#include "nmap_error.h"
 #include "xml.h"
 
+#include <math.h>
+#include <limits>
+
 extern NmapOps o;
 
 /* Call this function on a newly allocated struct timeout_info to
@@ -110,7 +83,7 @@ void initialize_timeout_info(struct timeout_info *to) {
   to->timeout = o.initialRttTimeout() * 1000;
 }
 
-/* Adjust our timeout values based on the time the latest probe took for a 
+/* Adjust our timeout values based on the time the latest probe took for a
    response.  We update our RTT averages, etc. */
 void adjust_timeouts(struct timeval sent, struct timeout_info *to) {
   struct timeval received;
@@ -124,9 +97,9 @@ void adjust_timeouts(struct timeval sent, struct timeout_info *to) {
  the receive time too (which could be because it was received a while
  back or it could be for efficiency because the caller already knows
  the current time */
-void adjust_timeouts2(const struct timeval *sent, 
-		      const struct timeval *received, 
-		      struct timeout_info *to) {
+void adjust_timeouts2(const struct timeval *sent,
+                      const struct timeval *received,
+                      struct timeout_info *to) {
   long delta = 0;
 
   if (o.debugging > 3) {
@@ -156,30 +129,30 @@ void adjust_timeouts2(const struct timeval *sent,
 
     if (delta >= 8000000 || delta < 0) {
       if (o.verbose)
-	error("%s: packet supposedly had rtt of %ld microseconds.  Ignoring time.", __func__, delta);
+        error("%s: packet supposedly had rtt of %ld microseconds.  Ignoring time.", __func__, delta);
       return;
     }
     rttdelta = delta - to->srtt;
     /* sanity check 2*/
     if (rttdelta > 1500000 && rttdelta > 3 * to->srtt + 2 * to->rttvar) {
       if (o.debugging) {
-	log_write(LOG_STDOUT, "Bogus rttdelta: %ld (srtt %d) ... ignoring\n", rttdelta, to->srtt);
+        log_write(LOG_STDOUT, "Bogus rttdelta: %ld (srtt %d) ... ignoring\n", rttdelta, to->srtt);
       }
       return;
     }
     to->srtt += rttdelta >> 3;
     to->rttvar += (ABS(rttdelta) - to->rttvar) >> 2;
-    to->timeout = to->srtt + (to->rttvar << 2);  
+    to->timeout = to->srtt + (to->rttvar << 2);
   }
   if (to->rttvar > 2300000) {
     error("RTTVAR has grown to over 2.3 seconds, decreasing to 2.0");
     to->rttvar = 2000000;
   }
-  
+
   /* It hurts to do this ... it really does ... but otherwise we are being
      too risky */
-  to->timeout = box(o.minRttTimeout() * 1000, o.maxRttTimeout() * 1000,  
-		    to->timeout);
+  to->timeout = box(o.minRttTimeout() * 1000, o.maxRttTimeout() * 1000,
+                    to->timeout);
 
   if (o.scan_delay)
     to->timeout = MAX((unsigned) to->timeout, o.scan_delay * 1000);
@@ -188,7 +161,7 @@ void adjust_timeouts2(const struct timeval *sent,
     log_write(LOG_STDOUT, "delta %ld ==> srtt: %d rttvar: %d to: %d\n", delta, to->srtt, to->rttvar, to->timeout);
   }
 
-  /* if (to->srtt < 0 || to->rttvar < 0 || to->timeout < 0 || delta < -50000000 || 
+  /* if (to->srtt < 0 || to->rttvar < 0 || to->timeout < 0 || delta < -50000000 ||
       sent->tv_sec == 0 || received->tv_sec == 0 ) {
     fatal("Serious time computation problem in adjust_timeout ... received = (%ld, %ld) sent=(%ld,%ld) delta = %ld srtt = %d rttvar = %d to = %d", (long) received->tv_sec, (long)received->tv_usec, (long) sent->tv_sec, (long) sent->tv_usec, delta, to->srtt, to->rttvar, to->timeout);
   } */
@@ -218,7 +191,7 @@ void enforce_scan_delay(struct timeval *tv) {
 
   gettimeofday(&now, NULL);
   time_diff = TIMEVAL_MSEC_SUBTRACT(now, lastcall);
-  if (time_diff < (int) o.scan_delay) {  
+  if (time_diff < (int) o.scan_delay) {
     if (o.debugging > 1) {
       log_write(LOG_PLAIN, "Sleeping for %d milliseconds in %s()\n", o.scan_delay - time_diff, __func__);
     }
@@ -230,7 +203,7 @@ void enforce_scan_delay(struct timeval *tv) {
     memcpy(tv, &lastcall, sizeof(struct timeval));
   }
 
-  return;    
+  return;
 }
 
 
@@ -405,7 +378,7 @@ void RateMeter::update(double amount, const struct timeval *now) {
     interval = MAX(current_rate_history, diff);
   else
     interval = TIMEVAL_SUBTRACT(*now, start_tv) / 1000000.0;
-  assert(diff <= interval);
+  assert(diff <= interval + std::numeric_limits<double>::epsilon());
   /* If we record an amount in the very same instant that the timer is started,
      there's no way to calculate meaningful rates. Ignore it. */
   if (interval == 0.0)
@@ -562,7 +535,7 @@ bool ScanProgressMeter::mayBePrinted(const struct timeval *now) {
       return true;
     else
       return false;
-  } 
+  }
 
   if (difftime(now->tv_sec, last_print_test.tv_sec) < 3)
     return false;  /* No point even checking too often */
@@ -592,8 +565,8 @@ static double estimate_time_left(double perc_done,
    so if mayBePrinted() is true, and it seems reasonable to do so
    because the estimate has changed significantly.  Returns whether
    or not a line was printed.*/
-bool ScanProgressMeter::printStatsIfNecessary(double perc_done, 
-					       const struct timeval *now) {
+bool ScanProgressMeter::printStatsIfNecessary(double perc_done,
+                                               const struct timeval *now) {
   struct timeval tvtmp;
   double time_left_s;
   bool printit = false;
@@ -636,18 +609,19 @@ bool ScanProgressMeter::printStatsIfNecessary(double perc_done,
 
   if (printit) {
      return printStats(perc_done, now);
-  } 
+  }
 
   return false;
 }
 
 /* Prints an estimate of when this scan will complete.  */
-bool ScanProgressMeter::printStats(double perc_done, 
+bool ScanProgressMeter::printStats(double perc_done,
                                    const struct timeval *now) {
   struct timeval tvtmp;
   double time_left_s;
   time_t timet;
-  struct tm *ltime;
+  struct tm ltime;
+  int err;
 
   if (!now) {
     gettimeofday(&tvtmp, NULL);
@@ -662,7 +636,13 @@ bool ScanProgressMeter::printStats(double perc_done,
   if (perc_done < 0.01) {
     log_write(LOG_STDOUT, "%s Timing: About %.2f%% done\n",
         scantypestr, perc_done * 100);
-    log_flush(LOG_STDOUT);
+    xml_open_start_tag("taskprogress");
+    xml_attribute("task", "%s", scantypestr);
+    xml_attribute("time", "%lu", (unsigned long) now->tv_sec);
+    xml_attribute("percent", "%.2f", perc_done * 100);
+    xml_close_empty_tag();
+    xml_newline();
+    log_flush(LOG_STDOUT|LOG_XML);
     return true;
   }
 
@@ -674,14 +654,23 @@ bool ScanProgressMeter::printStats(double perc_done,
 
   /* Get the estimated time of day at completion */
   timet = last_est.tv_sec;
-  ltime = localtime(&timet);
-  assert(ltime);
+  err = n_localtime(&timet, &ltime);
 
-  log_write(LOG_STDOUT, "%s Timing: About %.2f%% done; ETC: %02d:%02d (%.f:%02.f:%02.f remaining)\n",
-      scantypestr, perc_done * 100, ltime->tm_hour, ltime->tm_min,
-      floor(time_left_s / 60.0 / 60.0),
-      floor(fmod(time_left_s / 60.0, 60.0)),
-      floor(fmod(time_left_s, 60.0)));
+  if (!err) {
+    log_write(LOG_STDOUT, "%s Timing: About %.2f%% done; ETC: %02d:%02d (%.f:%02.f:%02.f remaining)\n",
+        scantypestr, perc_done * 100, ltime.tm_hour, ltime.tm_min,
+        floor(time_left_s / 60.0 / 60.0),
+        floor(fmod(time_left_s / 60.0, 60.0)),
+        floor(fmod(time_left_s, 60.0)));
+  }
+  else {
+    log_write(LOG_STDERR, "Timing error: n_localtime(%f): %s\n", (double) timet, strerror(err));
+    log_write(LOG_STDOUT, "%s Timing: About %.2f%% done; ETC: Unknown (%.f:%02.f:%02.f remaining)\n",
+        scantypestr, perc_done * 100,
+        floor(time_left_s / 60.0 / 60.0),
+        floor(fmod(time_left_s / 60.0, 60.0)),
+        floor(fmod(time_left_s, 60.0)));
+  }
   xml_open_start_tag("taskprogress");
   xml_attribute("task", "%s", scantypestr);
   xml_attribute("time", "%lu", (unsigned long) now->tv_sec);
@@ -691,7 +680,7 @@ bool ScanProgressMeter::printStats(double perc_done,
   xml_close_empty_tag();
   xml_newline();
   log_flush(LOG_STDOUT|LOG_XML);
- 
+
   return true;
 }
 
@@ -701,7 +690,8 @@ bool ScanProgressMeter::printStats(double perc_done,
    additional_info may be NULL if no additional information is necessary. */
 bool ScanProgressMeter::beginOrEndTask(const struct timeval *now, const char *additional_info, bool beginning) {
   struct timeval tvtmp;
-  struct tm *tm;
+  struct tm tm;
+  int err;
   time_t tv_sec;
 
   if (!o.verbose) {
@@ -714,9 +704,14 @@ bool ScanProgressMeter::beginOrEndTask(const struct timeval *now, const char *ad
   }
 
   tv_sec = now->tv_sec;
-  tm = localtime(&tv_sec);
+  err = n_localtime(&tv_sec, &tm);
+  if (err)
+    log_write(LOG_STDERR, "Timing error: n_localtime(%f): %s\n", (double) tv_sec, strerror(err));
   if (beginning) {
-    log_write(LOG_STDOUT, "Initiating %s at %02d:%02d", scantypestr, tm->tm_hour, tm->tm_min);
+    if (!err)
+      log_write(LOG_STDOUT, "Initiating %s at %02d:%02d", scantypestr, tm.tm_hour, tm.tm_min);
+    else
+      log_write(LOG_STDOUT, "Initiating %s", scantypestr);
     xml_open_start_tag("taskbegin");
     xml_attribute("task", "%s", scantypestr);
     xml_attribute("time", "%lu", (unsigned long) now->tv_sec);
@@ -728,7 +723,10 @@ bool ScanProgressMeter::beginOrEndTask(const struct timeval *now, const char *ad
     xml_close_empty_tag();
     xml_newline();
   } else {
-    log_write(LOG_STDOUT, "Completed %s at %02d:%02d, %.2fs elapsed", scantypestr, tm->tm_hour, tm->tm_min, TIMEVAL_MSEC_SUBTRACT(*now, begin) / 1000.0);
+    if (!err)
+      log_write(LOG_STDOUT, "Completed %s at %02d:%02d, %.2fs elapsed", scantypestr, tm.tm_hour, tm.tm_min, TIMEVAL_MSEC_SUBTRACT(*now, begin) / 1000.0);
+    else
+      log_write(LOG_STDOUT, "Completed %s, %.2fs elapsed", scantypestr, TIMEVAL_MSEC_SUBTRACT(*now, begin) / 1000.0);
     xml_open_start_tag("taskend");
     xml_attribute("task", "%s", scantypestr);
     xml_attribute("time", "%lu", (unsigned long) now->tv_sec);
