@@ -2,119 +2,58 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
-# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
-# * modify, and redistribute this software under certain conditions.  If    *
-# * you wish to embed Nmap technology into proprietary software, we sell    *
-# * alternative licenses (contact sales@nmap.com).  Dozens of software      *
-# * vendors already license Nmap technology such as host discovery, port    *
-# * scanning, OS detection, version detection, and the Nmap Scripting       *
-# * Engine.                                                                 *
+# * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derivative works",  *
-# * yet it does not provide a detailed definition of that term.  To avoid   *
-# * misunderstandings, we interpret that term as broadly as copyright law   *
-# * allows.  For example, we consider an application to constitute a        *
-# * derivative work for the purpose of this license if it does any of the   *
-# * following with any software or content covered by this license          *
-# * ("Covered Software"):                                                   *
+# * This program is distributed under the terms of the Nmap Public Source   *
+# * License (NPSL). The exact license text applying to a particular Nmap    *
+# * release or source code control revision is contained in the LICENSE     *
+# * file distributed with that version of Nmap or source code control       *
+# * revision. More Nmap copyright/legal information is available from       *
+# * https://nmap.org/book/man-legal.html, and further information on the    *
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+# * header summarizes some key points from the Nmap license, but is no      *
+# * substitute for the actual license text.                                 *
 # *                                                                         *
-# * o Integrates source code from Covered Software.                         *
+# * Nmap is generally free for end users to download and use themselves,    *
+# * including commercial use. It is available from https://nmap.org.        *
 # *                                                                         *
-# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
-# * or nmap-service-probes.                                                 *
+# * The Nmap license generally prohibits companies from using and           *
+# * redistributing Nmap in commercial products, but we sell a special Nmap  *
+# * OEM Edition with a more permissive license and special features for     *
+# * this purpose. See https://nmap.org/oem/                                 *
 # *                                                                         *
-# * o Is designed specifically to execute Covered Software and parse the    *
-# * results (as opposed to typical shell or execution-menu apps, which will *
-# * execute anything you tell them to).                                     *
+# * If you have received a written Nmap license agreement or contract       *
+# * stating terms other than these (such as an Nmap OEM license), you may   *
+# * choose to use and redistribute Nmap under those terms instead.          *
 # *                                                                         *
-# * o Includes Covered Software in a proprietary executable installer.  The *
-# * installers produced by InstallShield are an example of this.  Including *
-# * Nmap with other software in compressed or archival form does not        *
-# * trigger this provision, provided appropriate open source decompression  *
-# * or de-archiving software is widely available for no charge.  For the    *
-# * purposes of this license, an installer is considered to include Covered *
-# * Software even if it actually retrieves a copy of Covered Software from  *
-# * another source during runtime (such as by downloading it from the       *
-# * Internet).                                                              *
-# *                                                                         *
-# * o Links (statically or dynamically) to a library which does any of the  *
-# * above.                                                                  *
-# *                                                                         *
-# * o Executes a helper program, module, or script to do any of the above.  *
-# *                                                                         *
-# * This list is not exclusive, but is meant to clarify our interpretation  *
-# * of derived works with some common examples.  Other people may interpret *
-# * the plain GPL differently, so we consider this a special exception to   *
-# * the GPL that we apply to Covered Software.  Works which meet any of     *
-# * these conditions must conform to all of the terms of this license,      *
-# * particularly including the GPL Section 3 requirements of providing      *
-# * source code and allowing free redistribution of the work as a whole.    *
-# *                                                                         *
-# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two.                                  *
-# *                                                                         *
-# * Any redistribution of Covered Software, including any derived works,    *
-# * must obey and carry forward all of the terms of this license, including *
-# * obeying all GPL rules and restrictions.  For example, source code of    *
-# * the whole work must be provided and free redistribution must be         *
-# * allowed.  All GPL references to "this License", are to be treated as    *
-# * including the terms and conditions of this license text as well.        *
-# *                                                                         *
-# * Because this license imposes special exceptions to the GPL, Covered     *
-# * Work may not be combined (even as part of a larger work) with plain GPL *
-# * software.  The terms, conditions, and exceptions of this license must   *
-# * be included as well.  This license is incompatible with some other open *
-# * source licenses as well.  In some cases we can relicense portions of    *
-# * Nmap or grant special permissions to use it in other open source        *
-# * software.  Please contact fyodor@nmap.org with any such requests.       *
-# * Similarly, we don't incorporate incompatible open source software into  *
-# * Covered Software without special permission from the copyright holders. *
-# *                                                                         *
-# * If you have any questions about the licensing restrictions on using     *
-# * Nmap in other works, are happy to help.  As mentioned above, we also    *
-# * offer alternative license to integrate Nmap into proprietary            *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@nmap.com for further *
-# * information.                                                            *
-# *                                                                         *
-# * If you have received a written license agreement or contract for        *
-# * Covered Software stating terms other than these, you may choose to use  *
-# * and redistribute Covered Software under those terms instead of these.   *
+# * The official Nmap Windows builds include the Npcap software             *
+# * (https://npcap.com) for packet capture and transmission. It is under    *
+# * separate license terms which forbid redistribution without special      *
+# * permission. So the official Nmap Windows builds may not be              *
+# * redistributed without special permission (such as an Nmap OEM           *
+# * license).                                                               *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
 # * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to send your changes   *
-# * to the dev@nmap.org mailing list for possible incorporation into the    *
-# * main distribution.  By sending these changes to Fyodor or one of the    *
-# * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
-# * occasionally relicense the code to third parties as discussed above.    *
-# * If you wish to specify special license conditions of your               *
-# * contributions, just say so when you send them.                          *
+# * and add new features.  You are highly encouraged to submit your         *
+# * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+# * for possible incorporation into the main distribution. Unless you       *
+# * specify otherwise, it is understood that you are offering us very       *
+# * broad rights to use your submissions as described in the Nmap Public    *
+# * Source License Contributor Agreement. This is important because we      *
+# * fund the project by selling licenses with various terms, and also       *
+# * because the inability to relicense code has caused devastating          *
+# * problems for other Free Software projects (such as KDE and NASM).       *
 # *                                                                         *
-# * This program is distributed in the hope that it will be useful, but     *
-# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
-# * license file for more details (it's in a COPYING file included with     *
-# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
+# * The free version of Nmap is distributed in the hope that it will be     *
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+# * indemnification and commercial support are all available through the    *
+# * Npcap OEM program--see https://nmap.org/oem/                            *
 # *                                                                         *
 # ***************************************************************************/
 
@@ -122,12 +61,11 @@ import gtk
 import math
 import gobject
 
-import radialnet.util.drawing as drawing
 import radialnet.util.geometry as geometry
 
 from radialnet.bestwidgets.boxes import *
 from radialnet.core.Coordinate import PolarCoordinate
-from radialnet.gui.RadialNet import *
+import radialnet.gui.RadialNet as RadialNet
 from radialnet.bestwidgets.expanders import BWExpander
 
 
@@ -201,7 +139,7 @@ class ControlAction(BWExpander):
         try_set_tooltip_text(self.__jump_to, 'Change focus')
         self.__jump_to.connect('toggled',
                                self.__change_pointer,
-                               POINTER_JUMP_TO)
+                               RadialNet.POINTER_JUMP_TO)
 
         try:
             # gtk.STOCK_INFO is available only in PyGTK 2.8 and later.
@@ -214,20 +152,20 @@ class ControlAction(BWExpander):
         try_set_tooltip_text(self.__info, 'Show information')
         self.__info.connect('toggled',
                             self.__change_pointer,
-                            POINTER_INFO)
+                            RadialNet.POINTER_INFO)
 
         self.__group = gtk.RadioToolButton(self.__jump_to, gtk.STOCK_ADD)
         try_set_tooltip_text(self.__group, 'Group children')
         self.__group.connect('toggled',
                              self.__change_pointer,
-                             POINTER_GROUP)
+                             RadialNet.POINTER_GROUP)
 
         self.__region = gtk.RadioToolButton(self.__jump_to,
                                             gtk.STOCK_SELECT_COLOR)
         try_set_tooltip_text(self.__region, 'Fill region')
         self.__region.connect('toggled',
                               self.__change_pointer,
-                              POINTER_FILL)
+                              RadialNet.POINTER_FILL)
 
         self.__region_color = gtk.combo_box_new_text()
         self.__region_color.append_text(_('Red'))
@@ -256,7 +194,7 @@ class ControlAction(BWExpander):
         if pointer != self.radialnet.get_pointer_status():
             self.radialnet.set_pointer_status(pointer)
 
-        if pointer == POINTER_FILL:
+        if pointer == RadialNet.POINTER_FILL:
             self.__region_color.show()
         else:
             self.__region_color.hide()
@@ -393,7 +331,6 @@ class ControlVariableWidget(gtk.DrawingArea):
         self.context.set_dash([1, 0])
         self.context.set_font_size(10)
 
-        width = self.context.text_extents(self.__variable_name)[2]
         self.context.move_to(5, yc - self.__radius)
         self.context.show_text(self.__variable_name)
 
@@ -679,10 +616,10 @@ class ControlInterpolation(BWExpander):
                 self.__cartesian_radio, _('Polar'))
         self.__cartesian_radio.connect('toggled',
                                        self.__change_system,
-                                       INTERPOLATION_CARTESIAN)
+                                       RadialNet.INTERPOLATION_CARTESIAN)
         self.__polar_radio.connect('toggled',
                                    self.__change_system,
-                                   INTERPOLATION_POLAR)
+                                   RadialNet.INTERPOLATION_POLAR)
 
         self.__system_box = BWHBox()
         self.__system_box.bw_pack_start_noexpand_nofill(self.__polar_radio)
@@ -712,7 +649,7 @@ class ControlInterpolation(BWExpander):
         """
         active = self.radialnet.get_interpolation()
 
-        if active == INTERPOLATION_CARTESIAN:
+        if active == RadialNet.INTERPOLATION_CARTESIAN:
             self.__cartesian_radio.set_active(True)
 
         else:
@@ -727,7 +664,7 @@ class ControlInterpolation(BWExpander):
 
             active = self.radialnet.get_interpolation()
 
-            if active == INTERPOLATION_CARTESIAN:
+            if active == RadialNet.INTERPOLATION_CARTESIAN:
                 self.__cartesian_radio.set_active(True)
 
             else:
@@ -775,7 +712,7 @@ class ControlLayout(BWExpander):
     def __check_layout(self):
         """
         """
-        if self.__layout.get_active() == LAYOUT_WEIGHTED:
+        if self.__layout.get_active() == RadialNet.LAYOUT_WEIGHTED:
             self.__force.set_sensitive(True)
 
         else:
@@ -854,6 +791,8 @@ class ControlOptions(BWScrolledWindow):
 
         self.radialnet = radialnet
 
+        self.enable_labels = True
+
         self.__create_widgets()
 
     def __create_widgets(self):
@@ -878,12 +817,14 @@ class ControlOptions(BWScrolledWindow):
 
         self.__column_toggle = gtk.TreeViewColumn('', self.__cell_toggle)
         self.__column_toggle.add_attribute(self.__cell_toggle, 'active', 0)
+        self.__column_toggle.set_cell_data_func(self.__cell_toggle, self.__cell_toggle_data_method)
 
         self.__cell_text = gtk.CellRendererText()
 
         self.__column_text = gtk.TreeViewColumn(None,
                                                 self.__cell_text,
                                                 text=1)
+        self.__column_text.set_cell_data_func(self.__cell_text, self.__cell_text_data_method)
 
         self.__treeview = gtk.TreeView(self.__liststore)
         self.__treeview.set_enable_search(True)
@@ -896,13 +837,25 @@ class ControlOptions(BWScrolledWindow):
 
         gobject.timeout_add(REFRESH_RATE, self.__update_options)
 
+    def __cell_toggle_data_method(self, column, cell, model, it):
+        if not self.enable_labels and model.get_value(it, 1) == 'hostname':
+            cell.set_property('activatable', False)
+        else:
+            cell.set_property('activatable', True)
+
+    def __cell_text_data_method(self, column, cell, model, it):
+        if not self.enable_labels and model.get_value(it, 1) == 'hostname':
+            cell.set_property('strikethrough', True)
+        else:
+            cell.set_property('strikethrough', False)
+
     def __update_options(self):
         """
         """
         model = self.__liststore
 
         model[OPTIONS.index('address')][0] = self.radialnet.get_show_address()
-        model[OPTIONS.index('hostname')][0] = \
+        model[OPTIONS.index('hostname')][0] = self.enable_labels and \
                 self.radialnet.get_show_hostname()
         model[OPTIONS.index('icon')][0] = self.radialnet.get_show_icon()
         model[OPTIONS.index('latency')][0] = self.radialnet.get_show_latency()
@@ -921,6 +874,11 @@ class ControlOptions(BWScrolledWindow):
 
         if OPTIONS[option] == 'address':
             self.radialnet.set_show_address(model[option][0])
+            if model[option][0]:
+                model[OPTIONS.index('hostname')][0] = self.radialnet.get_show_hostname()
+            else:
+                model[OPTIONS.index('hostname')][0] = False
+            self.enable_labels = model[option][0]
 
         elif OPTIONS[option] == 'hostname':
             self.radialnet.set_show_hostname(model[option][0])
@@ -1039,7 +997,7 @@ class ControlNavigation(gtk.DrawingArea):
     def key_press(self, widget, event):
         """
         """
-        key = gtk.gdk.keyval_name(event.keyval)
+        # key = gtk.gdk.keyval_name(event.keyval)
 
         self.queue_draw()
 
@@ -1048,7 +1006,7 @@ class ControlNavigation(gtk.DrawingArea):
     def key_release(self, widget, event):
         """
         """
-        key = gtk.gdk.keyval_name(event.keyval)
+        # key = gtk.gdk.keyval_name(event.keyval)
 
         self.queue_draw()
 
