@@ -3,89 +3,58 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 with the clarifications and exceptions described  *
-# * below.  This guarantees your right to use, modify, and redistribute     *
-# * this software under certain conditions.  If you wish to embed Nmap      *
-# * technology into proprietary software, we sell alternative licenses      *
-# * (contact sales@insecure.com).  Dozens of software vendors already       *
-# * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, version detection, and the Nmap Scripting Engine.            *
+# * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derived works", yet *
-# * it does not provide a detailed definition of that term.  To avoid       *
-# * misunderstandings, we interpret that term as broadly as copyright law   *
-# * allows.  For example, we consider an application to constitute a        *
-# * "derivative work" for the purpose of this license if it does any of the *
-# * following:                                                              *
-# * o Integrates source code from Nmap                                      *
-# * o Reads or includes Nmap copyrighted data files, such as                *
-# *   nmap-os-db or nmap-service-probes.                                    *
-# * o Executes Nmap and parses the results (as opposed to typical shell or  *
-# *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                *
-# * o Integrates/includes/aggregates Nmap into a proprietary executable     *
-# *   installer, such as those produced by InstallShield.                   *
-# * o Links to a library or executes a program that does any of the above   *
+# * This program is distributed under the terms of the Nmap Public Source   *
+# * License (NPSL). The exact license text applying to a particular Nmap    *
+# * release or source code control revision is contained in the LICENSE     *
+# * file distributed with that version of Nmap or source code control       *
+# * revision. More Nmap copyright/legal information is available from       *
+# * https://nmap.org/book/man-legal.html, and further information on the    *
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+# * header summarizes some key points from the Nmap license, but is no      *
+# * substitute for the actual license text.                                 *
 # *                                                                         *
-# * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap, as well as other software we distribute under this       *
-# * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
-# * but is meant to clarify our interpretation of derived works with some   *
-# * common examples.  Our interpretation applies only to Nmap--we don't     *
-# * speak for other people's GPL works.                                     *
+# * Nmap is generally free for end users to download and use themselves,    *
+# * including commercial use. It is available from https://nmap.org.        *
 # *                                                                         *
-# * If you have any questions about the GPL licensing restrictions on using *
-# * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
-# * we also offer alternative license to integrate Nmap into proprietary    *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@insecure.com for     *
-# * further information.                                                    *
+# * The Nmap license generally prohibits companies from using and           *
+# * redistributing Nmap in commercial products, but we sell a special Nmap  *
+# * OEM Edition with a more permissive license and special features for     *
+# * this purpose. See https://nmap.org/oem/                                 *
 # *                                                                         *
-# * As a special exception to the GPL terms, Insecure.Com LLC grants        *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two. You must obey the GNU GPL in all *
-# * respects for all of the code used other than OpenSSL.  If you modify    *
-# * this file, you may extend this exception to your version of the file,   *
-# * but you are not obligated to do so.                                     *
+# * If you have received a written Nmap license agreement or contract       *
+# * stating terms other than these (such as an Nmap OEM license), you may   *
+# * choose to use and redistribute Nmap under those terms instead.          *
 # *                                                                         *
-# * If you received these files with a written license agreement or         *
-# * contract stating terms other than the terms above, then that            *
-# * alternative license agreement takes precedence over these comments.     *
+# * The official Nmap Windows builds include the Npcap software             *
+# * (https://npcap.com) for packet capture and transmission. It is under    *
+# * separate license terms which forbid redistribution without special      *
+# * permission. So the official Nmap Windows builds may not be              *
+# * redistributed without special permission (such as an Nmap OEM           *
+# * license).                                                               *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to send your changes   *
-# * to nmap-dev@insecure.org for possible incorporation into the main       *
-# * distribution.  By sending these changes to Fyodor or one of the         *
-# * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
-# * occasionally relicense the code to third parties as discussed above.    *
-# * If you wish to specify special license conditions of your               *
-# * contributions, just say so when you send them.                          *
+# * and add new features.  You are highly encouraged to submit your         *
+# * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+# * for possible incorporation into the main distribution. Unless you       *
+# * specify otherwise, it is understood that you are offering us very       *
+# * broad rights to use your submissions as described in the Nmap Public    *
+# * Source License Contributor Agreement. This is important because we      *
+# * fund the project by selling licenses with various terms, and also       *
+# * because the inability to relicense code has caused devastating          *
+# * problems for other Free Software projects (such as KDE and NASM).       *
 # *                                                                         *
-# * This program is distributed in the hope that it will be useful, but     *
-# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
-# * General Public License v2.0 for more details at                         *
-# * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
-# * included with Nmap.                                                     *
+# * The free version of Nmap is distributed in the hope that it will be     *
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+# * indemnification and commercial support are all available through the    *
+# * Npcap OEM program--see https://nmap.org/oem/                            *
 # *                                                                         *
 # ***************************************************************************/
 
@@ -97,20 +66,17 @@ import errno
 import locale
 import sys
 import os
-import re
 import tempfile
 import unittest
 
-import zenmapCore.I18N
+import zenmapCore.I18N  # lgtm[py/unused-import]
 
-from types import StringTypes
 try:
     import subprocess
 except ImportError, e:
     raise ImportError(str(e) + ".\n" + _("Python 2.4 or later is required."))
 
 import zenmapCore.Paths
-from zenmapCore.Paths import Path
 from zenmapCore.NmapOptions import NmapOptions
 from zenmapCore.UmitLogging import log
 from zenmapCore.UmitConf import PathsConfig
@@ -121,6 +87,7 @@ paths_config = PathsConfig()
 
 log.debug(">>> Platform: %s" % sys.platform)
 
+
 def wrap_file_in_preferred_encoding(f):
     """Wrap an open file to automatically decode its contents when reading from
     the encoding given by locale.getpreferredencoding, or just return the file
@@ -153,17 +120,19 @@ def wrap_file_in_preferred_encoding(f):
 
     return f
 
+
 def escape_nmap_filename(filename):
     """Escape '%' characters so they are not interpreted as strftime format
     specifiers, which are not supported by Zenmap."""
     return filename.replace("%", "%%")
 
+
 class NmapCommand(object):
     """This class represents an Nmap command line. It is responsible for
     starting, stopping, and returning the results from a command-line scan. A
     command line is represented as a string but it is split into a list of
     arguments for execution.
-    
+
     The normal output (stdout and stderr) are written to the file object
     self.stdout_file."""
 
@@ -200,7 +169,9 @@ class NmapCommand(object):
                 self.ops[op] = escape_nmap_filename(self.ops[op])
 
         if self.xml_is_temp:
-            self.xml_output_filename = tempfile.mktemp(prefix = APP_NAME + "-", suffix = ".xml")
+            fh, self.xml_output_filename = tempfile.mkstemp(
+                    prefix=APP_NAME + "-", suffix=".xml")
+            os.close(fh)
             self.ops["-oX"] = escape_nmap_filename(self.xml_output_filename)
 
         log.debug(">>> Temporary files:")
@@ -218,20 +189,31 @@ class NmapCommand(object):
 
     def kill(self):
         """Kill the nmap subprocess."""
+        from time import sleep
+
         log.debug(">>> Killing scan process %s" % self.command_process.pid)
 
         if sys.platform != "win32":
             try:
-                from signal import SIGKILL
-                os.kill(self.command_process.pid, SIGKILL)
-                self.command_process.wait()
-            except:
+                from signal import SIGTERM, SIGKILL
+                os.kill(self.command_process.pid, SIGTERM)
+                for i in range(10):
+                    sleep(0.5)
+                    if self.command_process.poll() is not None:
+                        # Process has been TERMinated
+                        break
+                else:
+                    log.debug(">>> SIGTERM has not worked even after waiting for 5 seconds. Using SIGKILL.")  # noqa
+                    os.kill(self.command_process.pid, SIGKILL)
+                    self.command_process.wait()
+            except Exception:
                 pass
         else:
             try:
                 import ctypes
-                ctypes.windll.kernel32.TerminateProcess(int(self.command_process._handle), -1)
-            except:
+                ctypes.windll.kernel32.TerminateProcess(
+                        int(self.command_process._handle), -1)
+            except Exception:
                 pass
 
     def get_path(self):
@@ -248,13 +230,13 @@ class NmapCommand(object):
                 search_paths.append(path)
         return os.pathsep.join(search_paths)
 
-    def run_scan(self, stderr = None):
+    def run_scan(self, stderr=None):
         """Run the command represented by this class."""
 
         # We don't need a file name for stdout output, just a handle. A
         # TemporaryFile is deleted as soon as it is closed, and in Unix is
         # unlinked immediately after creation so it's not even visible.
-        f = tempfile.TemporaryFile(mode = "rb", prefix = APP_NAME + "-stdout-")
+        f = tempfile.TemporaryFile(mode="rb", prefix=APP_NAME + "-stdout-")
         self.stdout_file = wrap_file_in_preferred_encoding(f)
         if stderr is None:
             stderr = f
@@ -272,7 +254,8 @@ class NmapCommand(object):
             # This keeps a terminal window from opening.
             startupinfo = subprocess.STARTUPINFO()
             try:
-                startupinfo.dwFlags |= subprocess._subprocess.STARTF_USESHOWWINDOW
+                startupinfo.dwFlags |= \
+                        subprocess._subprocess.STARTF_USESHOWWINDOW
             except AttributeError:
                 # This name is used before Python 2.6.5.
                 startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
@@ -281,7 +264,7 @@ class NmapCommand(object):
                                      stdin=subprocess.PIPE,
                                      stdout=f,
                                      stderr=stderr,
-                                     startupinfo = startupinfo,
+                                     startupinfo=startupinfo,
                                      env=env)
 
     def scan_state(self):
@@ -290,21 +273,24 @@ class NmapCommand(object):
         subprocess completed successfully. If the subprocess terminated with an
         error an exception is raised. The scan must have been started with
         run_scan before calling this method."""
-        if self.command_process == None:
+        if self.command_process is None:
             raise Exception("Scan is not running yet!")
 
         state = self.command_process.poll()
 
-        if state == None:
-            return True # True means that the process is still running
+        if state is None:
+            return True  # True means that the process is still running
         elif state == 0:
-            return False # False means that the process had a successful exit
+            return False  # False means that the process had a successful exit
         else:
             log.warning("An error occurred during the scan execution!")
-            log.warning("Command that raised the exception: '%s'" % self.ops.render_string())
+            log.warning("Command that raised the exception: '%s'" %
+                    self.ops.render_string())
             log.warning("Scan output:\n%s" % self.get_output())
 
-            raise Exception("An error occurred during the scan execution!\n\n'%s'" % self.get_output())
+            raise Exception(
+                    "An error occurred during the scan execution!\n\n'%s'" %
+                    self.get_output())
 
     def get_output(self):
         """Return the complete contents of the self.stdout_file. This modifies
@@ -317,4 +303,5 @@ class NmapCommand(object):
         return self.xml_output_filename
 
 if __name__ == '__main__':
-    unittest.TextTestRunner().run(unittest.TestLoader().loadTestsFromTestCase(SplitQuotedTest))
+    unittest.TextTestRunner().run(
+            unittest.TestLoader().loadTestsFromTestCase(SplitQuotedTest))
