@@ -3,99 +3,68 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 with the clarifications and exceptions described  *
-# * below.  This guarantees your right to use, modify, and redistribute     *
-# * this software under certain conditions.  If you wish to embed Nmap      *
-# * technology into proprietary software, we sell alternative licenses      *
-# * (contact sales@insecure.com).  Dozens of software vendors already       *
-# * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, version detection, and the Nmap Scripting Engine.            *
+# * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derived works", yet *
-# * it does not provide a detailed definition of that term.  To avoid       *
-# * misunderstandings, we interpret that term as broadly as copyright law   *
-# * allows.  For example, we consider an application to constitute a        *
-# * "derivative work" for the purpose of this license if it does any of the *
-# * following:                                                              *
-# * o Integrates source code from Nmap                                      *
-# * o Reads or includes Nmap copyrighted data files, such as                *
-# *   nmap-os-db or nmap-service-probes.                                    *
-# * o Executes Nmap and parses the results (as opposed to typical shell or  *
-# *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                *
-# * o Integrates/includes/aggregates Nmap into a proprietary executable     *
-# *   installer, such as those produced by InstallShield.                   *
-# * o Links to a library or executes a program that does any of the above   *
+# * This program is distributed under the terms of the Nmap Public Source   *
+# * License (NPSL). The exact license text applying to a particular Nmap    *
+# * release or source code control revision is contained in the LICENSE     *
+# * file distributed with that version of Nmap or source code control       *
+# * revision. More Nmap copyright/legal information is available from       *
+# * https://nmap.org/book/man-legal.html, and further information on the    *
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+# * header summarizes some key points from the Nmap license, but is no      *
+# * substitute for the actual license text.                                 *
 # *                                                                         *
-# * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap, as well as other software we distribute under this       *
-# * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
-# * but is meant to clarify our interpretation of derived works with some   *
-# * common examples.  Our interpretation applies only to Nmap--we don't     *
-# * speak for other people's GPL works.                                     *
+# * Nmap is generally free for end users to download and use themselves,    *
+# * including commercial use. It is available from https://nmap.org.        *
 # *                                                                         *
-# * If you have any questions about the GPL licensing restrictions on using *
-# * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
-# * we also offer alternative license to integrate Nmap into proprietary    *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@insecure.com for     *
-# * further information.                                                    *
+# * The Nmap license generally prohibits companies from using and           *
+# * redistributing Nmap in commercial products, but we sell a special Nmap  *
+# * OEM Edition with a more permissive license and special features for     *
+# * this purpose. See https://nmap.org/oem/                                 *
 # *                                                                         *
-# * As a special exception to the GPL terms, Insecure.Com LLC grants        *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two. You must obey the GNU GPL in all *
-# * respects for all of the code used other than OpenSSL.  If you modify    *
-# * this file, you may extend this exception to your version of the file,   *
-# * but you are not obligated to do so.                                     *
+# * If you have received a written Nmap license agreement or contract       *
+# * stating terms other than these (such as an Nmap OEM license), you may   *
+# * choose to use and redistribute Nmap under those terms instead.          *
 # *                                                                         *
-# * If you received these files with a written license agreement or         *
-# * contract stating terms other than the terms above, then that            *
-# * alternative license agreement takes precedence over these comments.     *
+# * The official Nmap Windows builds include the Npcap software             *
+# * (https://npcap.com) for packet capture and transmission. It is under    *
+# * separate license terms which forbid redistribution without special      *
+# * permission. So the official Nmap Windows builds may not be              *
+# * redistributed without special permission (such as an Nmap OEM           *
+# * license).                                                               *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to send your changes   *
-# * to nmap-dev@insecure.org for possible incorporation into the main       *
-# * distribution.  By sending these changes to Fyodor or one of the         *
-# * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
-# * occasionally relicense the code to third parties as discussed above.    *
-# * If you wish to specify special license conditions of your               *
-# * contributions, just say so when you send them.                          *
+# * and add new features.  You are highly encouraged to submit your         *
+# * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+# * for possible incorporation into the main distribution. Unless you       *
+# * specify otherwise, it is understood that you are offering us very       *
+# * broad rights to use your submissions as described in the Nmap Public    *
+# * Source License Contributor Agreement. This is important because we      *
+# * fund the project by selling licenses with various terms, and also       *
+# * because the inability to relicense code has caused devastating          *
+# * problems for other Free Software projects (such as KDE and NASM).       *
 # *                                                                         *
-# * This program is distributed in the hope that it will be useful, but     *
-# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
-# * General Public License v2.0 for more details at                         *
-# * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
-# * included with Nmap.                                                     *
+# * The free version of Nmap is distributed in the hope that it will be     *
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+# * indemnification and commercial support are all available through the    *
+# * Npcap OEM program--see https://nmap.org/oem/                            *
 # *                                                                         *
 # ***************************************************************************/
 
 import gtk
-import os.path
 import webbrowser
 
 from zenmapGUI.higwidgets.higdialogs import HIGDialog
 from zenmapGUI.higwidgets.higwindows import HIGWindow
-from zenmapGUI.higwidgets.higboxes import HIGVBox, HIGHBox, hig_box_space_holder
+from zenmapGUI.higwidgets.higboxes import HIGVBox, HIGHBox, \
+        hig_box_space_holder
 from zenmapGUI.higwidgets.higbuttons import HIGButton
 from zenmapGUI.higwidgets.hignotebooks import HIGNotebook
 from zenmapGUI.higwidgets.higscrollers import HIGScrolledWindow
@@ -104,12 +73,17 @@ from zenmapGUI.higwidgets.higtextviewers import HIGTextView
 from zenmapCore.Name import APP_DISPLAY_NAME, APP_WEB_SITE, APP_COPYRIGHT, \
     NMAP_DISPLAY_NAME, NMAP_WEB_SITE, UMIT_DISPLAY_NAME, UMIT_WEB_SITE
 from zenmapCore.Version import VERSION
-from zenmapCore.Paths import Path
-import zenmapCore.I18N
+import zenmapCore.I18N  # lgtm[py/unused-import]
+
+
+# Prevent loading PyXML
+import xml
+xml.__path__ = [x for x in xml.__path__ if "_xmlplus" not in x]
 
 # For escaping text in marked-up labels.
 from xml.sax.saxutils import escape
 
+
 class _program_entry(gtk.VBox):
     """A little box containing labels with a program's name and
     description and a clickable link to its web site."""
@@ -118,7 +92,7 @@ class _program_entry(gtk.VBox):
     # web site button.
     NAME_WEB_SITE_SPACING = 20
 
-    def __init__(self, name = None, web_site = None, description = None):
+    def __init__(self, name=None, web_site=None, description=None):
         gtk.VBox.__init__(self)
 
         self.hbox = gtk.HBox(False, self.NAME_WEB_SITE_SPACING)
@@ -126,7 +100,9 @@ class _program_entry(gtk.VBox):
 
         if name is not None:
             name_label = gtk.Label()
-            name_label.set_markup("<span size=\"large\" weight=\"bold\">%s</span>" % escape(name))
+            name_label.set_markup(
+                    '<span size="large" weight="bold">%s</span>' % escape(
+                        name))
             self.hbox.pack_start(name_label, False)
 
         if web_site is not None:
@@ -149,41 +125,44 @@ class _program_entry(gtk.VBox):
     def _link_button_open(self, widget):
         webbrowser.open(widget.get_uri())
 
+
 class About(HIGDialog):
     """An about dialog showing information about the program. It is meant to
     have roughly the same feel as gtk.AboutDialog."""
     def __init__(self):
         HIGDialog.__init__(self)
-        self.set_title(_("About %s and %s") % (NMAP_DISPLAY_NAME, APP_DISPLAY_NAME))
+        self.set_title(_("About %s and %s") % (
+            NMAP_DISPLAY_NAME, APP_DISPLAY_NAME))
 
         self.vbox.set_border_width(12)
         self.vbox.set_spacing(12)
 
         label = gtk.Label()
-        label.set_markup("<span size=\"xx-large\" weight=\"bold\">%s %s</span>" \
-% (escape(APP_DISPLAY_NAME), escape(VERSION)))
+        label.set_markup(
+                '<span size="xx-large" weight="bold">%s %s</span>' % (
+                    escape(APP_DISPLAY_NAME), escape(VERSION)))
         label.set_selectable(True)
         self.vbox.pack_start(label)
 
         label = gtk.Label()
-        label.set_markup("<span size=\"small\">%s</span>" \
-% (escape(APP_COPYRIGHT)))
+        label.set_markup(
+                '<span size="small">%s</span>' % (escape(APP_COPYRIGHT)))
         self.vbox.pack_start(label)
 
-        entry = _program_entry(NMAP_DISPLAY_NAME, NMAP_WEB_SITE, _("""\
-%s is a free and open source utility for network exploration and security \
-auditing.""") % NMAP_DISPLAY_NAME)
+        entry = _program_entry(NMAP_DISPLAY_NAME, NMAP_WEB_SITE, _(
+            "%s is a free and open source utility for network exploration "
+            "and security auditing.") % NMAP_DISPLAY_NAME)
         self.vbox.pack_start(entry)
 
-        entry = _program_entry(APP_DISPLAY_NAME, APP_WEB_SITE, _("""\
-%s is a multi-platform graphical %s frontend and results viewer. It was \
-originally derived from %s.""") \
-% (APP_DISPLAY_NAME, NMAP_DISPLAY_NAME, UMIT_DISPLAY_NAME))
+        entry = _program_entry(APP_DISPLAY_NAME, APP_WEB_SITE, _(
+            "%s is a multi-platform graphical %s frontend and results viewer. "
+            "It was originally derived from %s.") % (
+                APP_DISPLAY_NAME, NMAP_DISPLAY_NAME, UMIT_DISPLAY_NAME))
         self.vbox.pack_start(entry)
 
-        entry = _program_entry(UMIT_DISPLAY_NAME, UMIT_WEB_SITE, _("""\
-%s is an %s GUI created as part of the Nmap/Google Summer of Code program.""") \
-% (UMIT_DISPLAY_NAME, NMAP_DISPLAY_NAME))
+        entry = _program_entry(UMIT_DISPLAY_NAME, UMIT_WEB_SITE, _(
+            "%s is an %s GUI created as part of the Nmap/Google Summer "
+            "of Code program.") % (UMIT_DISPLAY_NAME, NMAP_DISPLAY_NAME))
         button = gtk.Button(_("%s credits") % UMIT_DISPLAY_NAME)
         button.connect("clicked", self._show_umit_credits)
         entry.hbox.pack_start(button, False)
@@ -215,17 +194,20 @@ originally derived from %s.""") \
             return
 
         self._umit_credits_dialog = UmitCredits()
+
         def credits_destroyed(widget):
             # Mark that the credits dialog has been destroyed.
             self._umit_credits_dialog = None
+
         self._umit_credits_dialog.connect("destroy", credits_destroyed)
         self._umit_credits_dialog.show_all()
 
+
 class UmitCredits(HIGWindow):
     def __init__(self):
         HIGWindow.__init__(self)
         self.set_title(_("%s credits") % UMIT_DISPLAY_NAME)
-        self.set_size_request(-1,250)
+        self.set_size_request(-1, 250)
         self.set_position(gtk.WIN_POS_CENTER)
 
         self.__create_widgets()
@@ -265,12 +247,18 @@ class UmitCredits(HIGWindow):
         self.hbox._pack_expand_fill(hig_box_space_holder())
         self.hbox._pack_noexpand_nofill(self.btn_close)
 
-        self.notebook.append_page(self.written_by_scroll, gtk.Label(_("Written by")))
-        self.notebook.append_page(self.design_scroll, gtk.Label(_("Design")))
-        self.notebook.append_page(self.soc2007_scroll, gtk.Label(_("SoC 2007")))
-        self.notebook.append_page(self.contributors_scroll, gtk.Label(_("Contributors")))
-        self.notebook.append_page(self.translation_scroll, gtk.Label(_("Translation")))
-        self.notebook.append_page(self.nokia_scroll, gtk.Label(_("Maemo")))
+        self.notebook.append_page(
+                self.written_by_scroll, gtk.Label(_("Written by")))
+        self.notebook.append_page(
+                self.design_scroll, gtk.Label(_("Design")))
+        self.notebook.append_page(
+                self.soc2007_scroll, gtk.Label(_("SoC 2007")))
+        self.notebook.append_page(
+                self.contributors_scroll, gtk.Label(_("Contributors")))
+        self.notebook.append_page(
+                self.translation_scroll, gtk.Label(_("Translation")))
+        self.notebook.append_page(
+                self.nokia_scroll, gtk.Label(_("Maemo")))
 
         self.written_by_scroll.add(self.written_by_text)
         self.written_by_text.set_wrap_mode(gtk.WRAP_NONE)
@@ -290,7 +278,7 @@ class UmitCredits(HIGWindow):
         self.nokia_scroll.add(self.nokia_text)
         self.nokia_text.set_wrap_mode(gtk.WRAP_NONE)
 
-        self.btn_close.connect('clicked', lambda x,y=None:self.destroy())
+        self.btn_close.connect('clicked', lambda x, y=None: self.destroy())
 
     def set_text(self):
         b = self.written_by_text.get_buffer()
