@@ -3,109 +3,83 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 with the clarifications and exceptions described  *
-# * below.  This guarantees your right to use, modify, and redistribute     *
-# * this software under certain conditions.  If you wish to embed Nmap      *
-# * technology into proprietary software, we sell alternative licenses      *
-# * (contact sales@insecure.com).  Dozens of software vendors already       *
-# * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, version detection, and the Nmap Scripting Engine.            *
+# * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derived works", yet *
-# * it does not provide a detailed definition of that term.  To avoid       *
-# * misunderstandings, we interpret that term as broadly as copyright law   *
-# * allows.  For example, we consider an application to constitute a        *
-# * "derivative work" for the purpose of this license if it does any of the *
-# * following:                                                              *
-# * o Integrates source code from Nmap                                      *
-# * o Reads or includes Nmap copyrighted data files, such as                *
-# *   nmap-os-db or nmap-service-probes.                                    *
-# * o Executes Nmap and parses the results (as opposed to typical shell or  *
-# *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                *
-# * o Integrates/includes/aggregates Nmap into a proprietary executable     *
-# *   installer, such as those produced by InstallShield.                   *
-# * o Links to a library or executes a program that does any of the above   *
+# * This program is distributed under the terms of the Nmap Public Source   *
+# * License (NPSL). The exact license text applying to a particular Nmap    *
+# * release or source code control revision is contained in the LICENSE     *
+# * file distributed with that version of Nmap or source code control       *
+# * revision. More Nmap copyright/legal information is available from       *
+# * https://nmap.org/book/man-legal.html, and further information on the    *
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+# * header summarizes some key points from the Nmap license, but is no      *
+# * substitute for the actual license text.                                 *
 # *                                                                         *
-# * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap, as well as other software we distribute under this       *
-# * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
-# * but is meant to clarify our interpretation of derived works with some   *
-# * common examples.  Our interpretation applies only to Nmap--we don't     *
-# * speak for other people's GPL works.                                     *
+# * Nmap is generally free for end users to download and use themselves,    *
+# * including commercial use. It is available from https://nmap.org.        *
 # *                                                                         *
-# * If you have any questions about the GPL licensing restrictions on using *
-# * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
-# * we also offer alternative license to integrate Nmap into proprietary    *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@insecure.com for     *
-# * further information.                                                    *
+# * The Nmap license generally prohibits companies from using and           *
+# * redistributing Nmap in commercial products, but we sell a special Nmap  *
+# * OEM Edition with a more permissive license and special features for     *
+# * this purpose. See https://nmap.org/oem/                                 *
 # *                                                                         *
-# * As a special exception to the GPL terms, Insecure.Com LLC grants        *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two. You must obey the GNU GPL in all *
-# * respects for all of the code used other than OpenSSL.  If you modify    *
-# * this file, you may extend this exception to your version of the file,   *
-# * but you are not obligated to do so.                                     *
+# * If you have received a written Nmap license agreement or contract       *
+# * stating terms other than these (such as an Nmap OEM license), you may   *
+# * choose to use and redistribute Nmap under those terms instead.          *
 # *                                                                         *
-# * If you received these files with a written license agreement or         *
-# * contract stating terms other than the terms above, then that            *
-# * alternative license agreement takes precedence over these comments.     *
+# * The official Nmap Windows builds include the Npcap software             *
+# * (https://npcap.com) for packet capture and transmission. It is under    *
+# * separate license terms which forbid redistribution without special      *
+# * permission. So the official Nmap Windows builds may not be              *
+# * redistributed without special permission (such as an Nmap OEM           *
+# * license).                                                               *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to send your changes   *
-# * to nmap-dev@insecure.org for possible incorporation into the main       *
-# * distribution.  By sending these changes to Fyodor or one of the         *
-# * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
-# * occasionally relicense the code to third parties as discussed above.    *
-# * If you wish to specify special license conditions of your               *
-# * contributions, just say so when you send them.                          *
+# * and add new features.  You are highly encouraged to submit your         *
+# * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+# * for possible incorporation into the main distribution. Unless you       *
+# * specify otherwise, it is understood that you are offering us very       *
+# * broad rights to use your submissions as described in the Nmap Public    *
+# * Source License Contributor Agreement. This is important because we      *
+# * fund the project by selling licenses with various terms, and also       *
+# * because the inability to relicense code has caused devastating          *
+# * problems for other Free Software projects (such as KDE and NASM).       *
 # *                                                                         *
-# * This program is distributed in the hope that it will be useful, but     *
-# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
-# * General Public License v2.0 for more details at                         *
-# * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
-# * included with Nmap.                                                     *
+# * The free version of Nmap is distributed in the hope that it will be     *
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+# * indemnification and commercial support are all available through the    *
+# * Npcap OEM program--see https://nmap.org/oem/                            *
 # *                                                                         *
 # ***************************************************************************/
 
 import gobject
 import gtk
-import pango
 import os
 import os.path
 import sys
+
+# Prevent loading PyXML
+import xml
+xml.__path__ = [x for x in xml.__path__ if "_xmlplus" not in x]
+
 import xml.sax
 
-from zenmapGUI.higwidgets.higdialogs import HIGAlertDialog, HIGDialog
-from zenmapGUI.higwidgets.higboxes import HIGVBox, HIGHBox, hig_box_space_holder
+from zenmapGUI.higwidgets.higdialogs import HIGAlertDialog
+from zenmapGUI.higwidgets.higboxes import HIGVBox, HIGHBox, \
+    hig_box_space_holder
 from zenmapGUI.higwidgets.higlabels import HIGSectionLabel
 from zenmapGUI.higwidgets.higtables import HIGTable
 from zenmapGUI.higwidgets.higbuttons import HIGButton
 
 from zenmapCore.NmapParser import NmapParser
 from zenmapCore.UmitLogging import log
-import zenmapCore.I18N
+import zenmapCore.I18N  # lgtm[py/unused-import]
 import zenmapCore.Diff
 
 from zenmapGUI.FileChoosers import ResultsFileSingleChooserDialog
@@ -113,6 +87,7 @@ from zenmapGUI.FileChoosers import ResultsFileSingleChooserDialog
 # In milliseconds.
 NDIFF_CHECK_TIMEOUT = 200
 
+
 class ScanChooser(HIGVBox):
     """This class allows the selection of scan results from the list of open
     tabs or from a file. It emits the "changed" signal when the scan selection
@@ -162,22 +137,23 @@ class ScanChooser(HIGVBox):
     def get_buffer(self):
         return self.txt_scan_result.get_buffer()
 
-    def show_scan (self, widget):
+    def show_scan(self, widget):
         nmap_output = self.get_nmap_output()
-        if nmap_output is not None:
+        if nmap_output:
             self.txt_scan_result.get_buffer().set_text(nmap_output)
 
     def normalize_output(self, output):
         return "\n".join(output.split("\\n"))
 
-    def _pack_hbox (self):
+    def _pack_hbox(self):
         self.hbox._pack_noexpand_nofill(hig_box_space_holder())
         self.hbox._pack_expand_fill(self.table)
 
-    def _attaching_widgets (self):
-        self.table.attach(self.combo_scan, 0,1,0,1, yoptions=0)
-        self.table.attach(self.btn_open_scan, 1,2,0,1, yoptions=0, xoptions=0)
-        self.table.attach(self.exp_scan, 0,2,1,2)
+    def _attaching_widgets(self):
+        self.table.attach(self.combo_scan, 0, 1, 0, 1, yoptions=0)
+        self.table.attach(
+            self.btn_open_scan, 1, 2, 0, 1, yoptions=0, xoptions=0)
+        self.table.attach(self.exp_scan, 0, 2, 1, 2)
 
     def _set_scrolled(self):
         self.scrolled.set_border_width(5)
@@ -192,19 +168,20 @@ class ScanChooser(HIGVBox):
         # Setting scrolled window
         self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 
-    def _set_text_view (self):
+    def _set_text_view(self):
         self.txg_table = self.txt_scan_result.get_buffer().get_tag_table()
         self.txg_table.add(self.txg_tag)
         self.txg_tag.set_property("family", "Monospace")
 
         self.txt_scan_result.set_wrap_mode(gtk.WRAP_WORD)
         self.txt_scan_result.set_editable(False)
-        self.txt_scan_result.get_buffer().connect("changed", self._text_changed_cb)
+        self.txt_scan_result.get_buffer().connect(
+            "changed", self._text_changed_cb)
 
-    def _set_open_button (self):
+    def _set_open_button(self):
         self.btn_open_scan.connect('clicked', self.open_file)
 
-    def open_file (self, widget):
+    def open_file(self, widget):
         file_chooser = ResultsFileSingleChooserDialog(_("Select Scan Result"))
 
         response = file_chooser.run()
@@ -214,18 +191,22 @@ class ScanChooser(HIGVBox):
             try:
                 parser = NmapParser()
                 parser.parse_file(file_chosen)
-            except xml.sax.SAXParseException, e:
+            except xml.sax.SAXParseException as e:
                 alert = HIGAlertDialog(
                     message_format='<b>%s</b>' % _('Error parsing file'),
-                    secondary_text=_("The file is not an Nmap XML output file. \
-The parsing error that occurred was\n%s") % str(e))
+                    secondary_text=_(
+                        "The file is not an Nmap XML output file. "
+                        "The parsing error that occurred was\n%s") % str(e))
                 alert.run()
                 alert.destroy()
                 return False
-            except Exception, e:
+            except Exception as e:
                 alert = HIGAlertDialog(
-                        message_format='<b>%s</b>' % _('Cannot open selected file'),
-                        secondary_text=_("This error occurred while trying to open the file:\n%s") % str(e))
+                    message_format='<b>%s</b>' % _(
+                        'Cannot open selected file'),
+                    secondary_text=_("""\
+                        This error occurred while trying to open the file:
+                        %s""") % str(e))
                 alert.run()
                 alert.destroy()
                 return False
@@ -245,26 +226,24 @@ The parsing error that occurred was\n%s") % str(e))
         self.list_scan.append([new_scan_name])
         self.scan_dict[new_scan_name] = parser
 
-    def _text_changed_cb (self, widget):
-        buff = self.txt_scan_result.get_buffer ()
-        buff.apply_tag(self.txg_tag, buff.get_start_iter(), buff.get_end_iter())
+    def _text_changed_cb(self, widget):
+        buff = self.txt_scan_result.get_buffer()
+        buff.apply_tag(
+            self.txg_tag, buff.get_start_iter(), buff.get_end_iter())
 
     def get_parsed_scan(self):
         """Return the currently selected scan's parsed output as an NmapParser
         object, or None if no valid scan is selected."""
         selected_scan = self.combo_scan.child.get_text()
-        if selected_scan in self.scan_dict:
-            return self.scan_dict[selected_scan]
-        # What's typed in the entry doesn't match a registered scan.
-        return None
+        return self.scan_dict.get(selected_scan)
 
     def get_nmap_output(self):
         """Return the currently selected scan's output as a string, or None if
         no valid scan is selected."""
-        parsed = self.parsed_scan
-        if parsed is not None:
-            return parsed.nmap_output
-        return None
+        if self.parsed_scan is not None:
+            return self.parsed_scan.get_nmap_output()
+        else:
+            return None
 
     nmap_output = property(get_nmap_output)
     parsed_scan = property(get_parsed_scan)
@@ -275,9 +254,10 @@ class DiffWindow(gtk.Window):
         gtk.Window.__init__(self)
         self.set_title(_("Compare Results"))
         self.ndiff_process = None
-        # We allow the user to start a new diff before the old one has finished.
-        # We have to keep references to old processes until they finish to avoid
-        # problems when tearing down the Python interpreter at program exit.
+        # We allow the user to start a new diff before the old one has
+        # finished.  We have to keep references to old processes until they
+        # finish to avoid problems when tearing down the Python interpreter at
+        # program exit.
         self.old_processes = []
         self.timer_id = None
 
@@ -327,12 +307,13 @@ class DiffWindow(gtk.Window):
         self.scan_chooser_a.connect('changed', self.refresh_diff)
         self.scan_chooser_b.connect('changed', self.refresh_diff)
 
-    def refresh_diff (self, widget):
+    def refresh_diff(self, widget):
         """This method is called whenever the diff output might have changed,
         such as when a different scan was selected in one of the choosers."""
         log.debug("Refresh diff.")
 
-        if self.ndiff_process is not None and self.ndiff_process.poll() is None:
+        if (self.ndiff_process is not None and
+                self.ndiff_process.poll() is None):
             # Put this in the list of old processes we keep track of.
             self.old_processes.append(self.ndiff_process)
             self.ndiff_process = None
@@ -345,16 +326,19 @@ class DiffWindow(gtk.Window):
         else:
             try:
                 self.ndiff_process = zenmapCore.Diff.ndiff(scan_a, scan_b)
-            except OSError, e:
+            except OSError as e:
                 alert = HIGAlertDialog(
-                    message_format = _("Error running ndiff"),
-                    secondary_text = _("There was an error running the ndiff program.\n\n") + str(e).decode(sys.getdefaultencoding(), "replace"))
+                    message_format=_("Error running ndiff"),
+                    secondary_text=_(
+                        "There was an error running the ndiff program.\n\n"
+                        ) + str(e).decode(sys.getdefaultencoding(), "replace"))
                 alert.run()
                 alert.destroy()
             else:
                 self.progress.show()
                 if self.timer_id is None:
-                    self.timer_id = gobject.timeout_add(NDIFF_CHECK_TIMEOUT, self.check_ndiff_process)
+                    self.timer_id = gobject.timeout_add(
+                        NDIFF_CHECK_TIMEOUT, self.check_ndiff_process)
 
     def check_ndiff_process(self):
         """Check if the ndiff subprocess is done and show the diff if it is.
@@ -379,23 +363,25 @@ class DiffWindow(gtk.Window):
                 # Successful completion.
                 try:
                     diff = self.ndiff_process.get_scan_diff()
-                except zenmapCore.Diff.NdiffParseException, e:
+                except zenmapCore.Diff.NdiffParseException as e:
                     alert = HIGAlertDialog(
-                        message_format = _("Error parsing ndiff output"),
-                        secondary_text = str(e))
+                        message_format=_("Error parsing ndiff output"),
+                        secondary_text=str(e))
                     alert.run()
                     alert.destroy()
                 else:
                     self.diff_view.show_diff(diff)
             else:
                 # Unsuccessful completion.
-                error_text = _("The ndiff process terminated with status code %d.") % status
+                error_text = _(
+                    "The ndiff process terminated with status code %d."
+                    ) % status
                 stderr = self.ndiff_process.stderr.read()
                 if len(stderr) > 0:
                     error_text += "\n\n" + stderr
                 alert = HIGAlertDialog(
-                    message_format = _("Error running ndiff"),
-                    secondary_text = error_text)
+                    message_format=_("Error running ndiff"),
+                    secondary_text=error_text)
                 alert.run()
                 alert.destroy()
 
@@ -414,6 +400,7 @@ class DiffWindow(gtk.Window):
     def close(self, widget=None, extra=None):
         self.destroy()
 
+
 class DiffView(gtk.TextView):
     REMOVE_COLOR = "#ffaaaa"
     ADD_COLOR = "#ccffcc"
@@ -425,9 +412,10 @@ class DiffView(gtk.TextView):
 
         buff = self.get_buffer()
         # Create text markup tags.
-        buff.create_tag("=", font = "Monospace")
-        buff.create_tag("-", font = "Monospace", background = self.REMOVE_COLOR)
-        buff.create_tag("+", font = "Monospace", background = self.ADD_COLOR)
+        buff.create_tag("=", font="Monospace")
+        buff.create_tag(
+            "-", font="Monospace", background=self.REMOVE_COLOR)
+        buff.create_tag("+", font="Monospace", background=self.ADD_COLOR)
 
     def clear(self):
         self.get_buffer().set_text(u"")
@@ -463,6 +451,6 @@ if __name__ == "__main__":
                      "Parsed 4": parsed4})
 
     dw.show_all()
-    dw.connect("delete-event", lambda x,y: gtk.main_quit())
+    dw.connect("delete-event", lambda x, y: gtk.main_quit())
 
     gtk.main()
