@@ -3,119 +3,58 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
-# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
-# * modify, and redistribute this software under certain conditions.  If    *
-# * you wish to embed Nmap technology into proprietary software, we sell    *
-# * alternative licenses (contact sales@nmap.com).  Dozens of software      *
-# * vendors already license Nmap technology such as host discovery, port    *
-# * scanning, OS detection, version detection, and the Nmap Scripting       *
-# * Engine.                                                                 *
+# * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derivative works",  *
-# * yet it does not provide a detailed definition of that term.  To avoid   *
-# * misunderstandings, we interpret that term as broadly as copyright law   *
-# * allows.  For example, we consider an application to constitute a        *
-# * derivative work for the purpose of this license if it does any of the   *
-# * following with any software or content covered by this license          *
-# * ("Covered Software"):                                                   *
+# * This program is distributed under the terms of the Nmap Public Source   *
+# * License (NPSL). The exact license text applying to a particular Nmap    *
+# * release or source code control revision is contained in the LICENSE     *
+# * file distributed with that version of Nmap or source code control       *
+# * revision. More Nmap copyright/legal information is available from       *
+# * https://nmap.org/book/man-legal.html, and further information on the    *
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+# * header summarizes some key points from the Nmap license, but is no      *
+# * substitute for the actual license text.                                 *
 # *                                                                         *
-# * o Integrates source code from Covered Software.                         *
+# * Nmap is generally free for end users to download and use themselves,    *
+# * including commercial use. It is available from https://nmap.org.        *
 # *                                                                         *
-# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
-# * or nmap-service-probes.                                                 *
+# * The Nmap license generally prohibits companies from using and           *
+# * redistributing Nmap in commercial products, but we sell a special Nmap  *
+# * OEM Edition with a more permissive license and special features for     *
+# * this purpose. See https://nmap.org/oem/                                 *
 # *                                                                         *
-# * o Is designed specifically to execute Covered Software and parse the    *
-# * results (as opposed to typical shell or execution-menu apps, which will *
-# * execute anything you tell them to).                                     *
+# * If you have received a written Nmap license agreement or contract       *
+# * stating terms other than these (such as an Nmap OEM license), you may   *
+# * choose to use and redistribute Nmap under those terms instead.          *
 # *                                                                         *
-# * o Includes Covered Software in a proprietary executable installer.  The *
-# * installers produced by InstallShield are an example of this.  Including *
-# * Nmap with other software in compressed or archival form does not        *
-# * trigger this provision, provided appropriate open source decompression  *
-# * or de-archiving software is widely available for no charge.  For the    *
-# * purposes of this license, an installer is considered to include Covered *
-# * Software even if it actually retrieves a copy of Covered Software from  *
-# * another source during runtime (such as by downloading it from the       *
-# * Internet).                                                              *
-# *                                                                         *
-# * o Links (statically or dynamically) to a library which does any of the  *
-# * above.                                                                  *
-# *                                                                         *
-# * o Executes a helper program, module, or script to do any of the above.  *
-# *                                                                         *
-# * This list is not exclusive, but is meant to clarify our interpretation  *
-# * of derived works with some common examples.  Other people may interpret *
-# * the plain GPL differently, so we consider this a special exception to   *
-# * the GPL that we apply to Covered Software.  Works which meet any of     *
-# * these conditions must conform to all of the terms of this license,      *
-# * particularly including the GPL Section 3 requirements of providing      *
-# * source code and allowing free redistribution of the work as a whole.    *
-# *                                                                         *
-# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two.                                  *
-# *                                                                         *
-# * Any redistribution of Covered Software, including any derived works,    *
-# * must obey and carry forward all of the terms of this license, including *
-# * obeying all GPL rules and restrictions.  For example, source code of    *
-# * the whole work must be provided and free redistribution must be         *
-# * allowed.  All GPL references to "this License", are to be treated as    *
-# * including the terms and conditions of this license text as well.        *
-# *                                                                         *
-# * Because this license imposes special exceptions to the GPL, Covered     *
-# * Work may not be combined (even as part of a larger work) with plain GPL *
-# * software.  The terms, conditions, and exceptions of this license must   *
-# * be included as well.  This license is incompatible with some other open *
-# * source licenses as well.  In some cases we can relicense portions of    *
-# * Nmap or grant special permissions to use it in other open source        *
-# * software.  Please contact fyodor@nmap.org with any such requests.       *
-# * Similarly, we don't incorporate incompatible open source software into  *
-# * Covered Software without special permission from the copyright holders. *
-# *                                                                         *
-# * If you have any questions about the licensing restrictions on using     *
-# * Nmap in other works, are happy to help.  As mentioned above, we also    *
-# * offer alternative license to integrate Nmap into proprietary            *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@nmap.com for further *
-# * information.                                                            *
-# *                                                                         *
-# * If you have received a written license agreement or contract for        *
-# * Covered Software stating terms other than these, you may choose to use  *
-# * and redistribute Covered Software under those terms instead of these.   *
+# * The official Nmap Windows builds include the Npcap software             *
+# * (https://npcap.com) for packet capture and transmission. It is under    *
+# * separate license terms which forbid redistribution without special      *
+# * permission. So the official Nmap Windows builds may not be              *
+# * redistributed without special permission (such as an Nmap OEM           *
+# * license).                                                               *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
 # * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to send your changes   *
-# * to the dev@nmap.org mailing list for possible incorporation into the    *
-# * main distribution.  By sending these changes to Fyodor or one of the    *
-# * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
-# * occasionally relicense the code to third parties as discussed above.    *
-# * If you wish to specify special license conditions of your               *
-# * contributions, just say so when you send them.                          *
+# * and add new features.  You are highly encouraged to submit your         *
+# * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+# * for possible incorporation into the main distribution. Unless you       *
+# * specify otherwise, it is understood that you are offering us very       *
+# * broad rights to use your submissions as described in the Nmap Public    *
+# * Source License Contributor Agreement. This is important because we      *
+# * fund the project by selling licenses with various terms, and also       *
+# * because the inability to relicense code has caused devastating          *
+# * problems for other Free Software projects (such as KDE and NASM).       *
 # *                                                                         *
-# * This program is distributed in the hope that it will be useful, but     *
-# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
-# * license file for more details (it's in a COPYING file included with     *
-# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
+# * The free version of Nmap is distributed in the hope that it will be     *
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+# * indemnification and commercial support are all available through the    *
+# * Npcap OEM program--see https://nmap.org/oem/                            *
 # *                                                                         *
 # ***************************************************************************/
 
@@ -123,7 +62,7 @@ import gtk
 
 import sys
 import os
-from os.path import split, isfile, join, abspath, exists
+from os.path import split, isfile, join, abspath
 
 # Prevent loading PyXML
 import xml
@@ -131,16 +70,15 @@ xml.__path__ = [x for x in xml.__path__ if "_xmlplus" not in x]
 
 import xml.sax.saxutils
 
-from time import time
-
 from zenmapGUI.higwidgets.higwindows import HIGMainWindow
 from zenmapGUI.higwidgets.higdialogs import HIGDialog, HIGAlertDialog
 from zenmapGUI.higwidgets.higlabels import HIGEntryLabel
 from zenmapGUI.higwidgets.higboxes import HIGHBox, HIGVBox
-from zenmapGUI.higwidgets.higlabels import HIGSectionLabel
 
 import zenmapGUI.App
-from zenmapGUI.FileChoosers import *
+from zenmapGUI.FileChoosers import RESPONSE_OPEN_DIRECTORY, \
+        ResultsFileChooserDialog, SaveResultsFileChooserDialog, \
+        SaveToDirectoryChooserDialog
 from zenmapGUI.ScanInterface import ScanInterface
 from zenmapGUI.ProfileEditor import ProfileEditor
 from zenmapGUI.About import About
@@ -153,10 +91,9 @@ from zenmapCore.BasePaths import fs_enc
 from zenmapCore.Paths import Path
 from zenmapCore.RecentScans import recent_scans
 from zenmapCore.UmitLogging import log
-import zenmapCore.I18N
+import zenmapCore.I18N  # lgtm[py/unused-import]
 import zenmapGUI.Print
-from zenmapCore.UmitConf import SearchConfig, is_maemo, WindowConfig
-from zenmapCore.NetworkInventory import FilteredNetworkInventory
+from zenmapCore.UmitConf import SearchConfig, is_maemo, WindowConfig, config_parser
 
 UmitScanWindow = None
 hildon = None
@@ -243,7 +180,7 @@ class ScanWindow(UmitScanWindow):
         # gtk.STOCK_ABOUT is only available in PyGTK 2.6 and later.
         try:
             about_icon = gtk.STOCK_ABOUT
-        except:
+        except AttributeError:
             about_icon = None
 
         self.main_actions = [
@@ -498,8 +435,8 @@ class ScanWindow(UmitScanWindow):
                         xml.sax.saxutils.quoteattr(scan))
 
                 self.main_actions.append(new_rscan)
-        else:
-            new_rscan_xml += "<separator />\n"
+
+        new_rscan_xml += "<separator />\n"
 
         self.default_ui %= new_rscan_xml
 
@@ -759,9 +696,18 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
             scan_interface.saved_filename = directory
 
             # Saving recent scan information
-            for filename in filenames:
-                recent_scans.add_recent_scan(filename)
-            recent_scans.save()
+            try:
+                for filename in filenames:
+                    recent_scans.add_recent_scan(filename)
+                recent_scans.save()
+            except (OSError, IOError), e:
+                alert = HIGAlertDialog(
+                        message_format=_(
+                            "Can't save recent scan information"),
+                        secondary_text=_(
+                            "Can't open file to write.\n%s") % str(e))
+                alert.run()
+                alert.destroy()
 
     def _save(self, scan_interface, saved_filename, selected_index,
             format="xml"):
@@ -786,8 +732,17 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
 
             if format == "xml":
                 # Saving recent scan information
-                recent_scans.add_recent_scan(saved_filename)
-                recent_scans.save()
+                try:
+                    recent_scans.add_recent_scan(saved_filename)
+                    recent_scans.save()
+                except (OSError, IOError), e:
+                    alert = HIGAlertDialog(
+                            message_format=_(
+                                "Can't save recent scan information"),
+                            secondary_text=_(
+                                "Can't open file to write.\n%s") % str(e))
+                    alert.run()
+                    alert.destroy()
 
     def get_empty_interface(self):
         """Return this window if it is empty, otherwise create and return a new
@@ -908,6 +863,16 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
         window.x, window.y = self.get_position()
         window.width, window.height = self.get_size()
         window.save_changes()
+        if config_parser.failed:
+            alert = HIGAlertDialog(
+                    message_format=_("Can't save Zenmap configuration"),
+                    # newline before path to help avoid weird line wrapping
+                    secondary_text=_(
+                        'An error occurred when saving to\n%s'
+                        '\nThe error was: %s.'
+                        ) % (Path.user_config_file, config_parser.failed))
+            alert.run()
+            alert.destroy()
 
         self.destroy()
 
