@@ -3,128 +3,104 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 with the clarifications and exceptions described  *
-# * below.  This guarantees your right to use, modify, and redistribute     *
-# * this software under certain conditions.  If you wish to embed Nmap      *
-# * technology into proprietary software, we sell alternative licenses      *
-# * (contact sales@insecure.com).  Dozens of software vendors already       *
-# * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, and version detection.                                       *
+# * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derived works", yet *
-# * it does not provide a detailed definition of that term.  To avoid       *
-# * misunderstandings, we consider an application to constitute a           *
-# * "derivative work" for the purpose of this license if it does any of the *
-# * following:                                                              *
-# * o Integrates source code from Nmap                                      *
-# * o Reads or includes Nmap copyrighted data files, such as                *
-# *   nmap-os-db or nmap-service-probes.                                    *
-# * o Executes Nmap and parses the results (as opposed to typical shell or  *
-# *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                *
-# * o Integrates/includes/aggregates Nmap into a proprietary executable     *
-# *   installer, such as those produced by InstallShield.                   *
-# * o Links to a library or executes a program that does any of the above   *
+# * This program is distributed under the terms of the Nmap Public Source   *
+# * License (NPSL). The exact license text applying to a particular Nmap    *
+# * release or source code control revision is contained in the LICENSE     *
+# * file distributed with that version of Nmap or source code control       *
+# * revision. More Nmap copyright/legal information is available from       *
+# * https://nmap.org/book/man-legal.html, and further information on the    *
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+# * header summarizes some key points from the Nmap license, but is no      *
+# * substitute for the actual license text.                                 *
 # *                                                                         *
-# * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap.  This list is not exclusive, but is meant to clarify our *
-# * interpretation of derived works with some common examples.  Our         *
-# * interpretation applies only to Nmap--we don't speak for other people's  *
-# * GPL works.                                                              *
+# * Nmap is generally free for end users to download and use themselves,    *
+# * including commercial use. It is available from https://nmap.org.        *
 # *                                                                         *
-# * If you have any questions about the GPL licensing restrictions on using *
-# * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
-# * we also offer alternative license to integrate Nmap into proprietary    *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates as well as helping to     *
-# * fund the continued development of Nmap technology.  Please email        *
-# * sales@insecure.com for further information.                             *
+# * The Nmap license generally prohibits companies from using and           *
+# * redistributing Nmap in commercial products, but we sell a special Nmap  *
+# * OEM Edition with a more permissive license and special features for     *
+# * this purpose. See https://nmap.org/oem/                                 *
 # *                                                                         *
-# * As a special exception to the GPL terms, Insecure.Com LLC grants        *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two. You must obey the GNU GPL in all *
-# * respects for all of the code used other than OpenSSL.  If you modify    *
-# * this file, you may extend this exception to your version of the file,   *
-# * but you are not obligated to do so.                                     *
+# * If you have received a written Nmap license agreement or contract       *
+# * stating terms other than these (such as an Nmap OEM license), you may   *
+# * choose to use and redistribute Nmap under those terms instead.          *
 # *                                                                         *
-# * If you received these files with a written license agreement or         *
-# * contract stating terms other than the terms above, then that            *
-# * alternative license agreement takes precedence over these comments.     *
+# * The official Nmap Windows builds include the Npcap software             *
+# * (https://npcap.com) for packet capture and transmission. It is under    *
+# * separate license terms which forbid redistribution without special      *
+# * permission. So the official Nmap Windows builds may not be              *
+# * redistributed without special permission (such as an Nmap OEM           *
+# * license).                                                               *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to send your changes   *
-# * to nmap-dev@insecure.org for possible incorporation into the main       *
-# * distribution.  By sending these changes to Fyodor or one of the         *
-# * Insecure.Org development mailing lists, it is assumed that you are      *
-# * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
-# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
-# * will always be available Open Source, but this is important because the *
-# * inability to relicense code has caused devastating problems for other   *
-# * Free Software projects (such as KDE and NASM).  We also occasionally    *
-# * relicense the code to third parties as discussed above.  If you wish to *
-# * specify special license conditions of your contributions, just say so   *
-# * when you send them.                                                     *
+# * and add new features.  You are highly encouraged to submit your         *
+# * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+# * for possible incorporation into the main distribution. Unless you       *
+# * specify otherwise, it is understood that you are offering us very       *
+# * broad rights to use your submissions as described in the Nmap Public    *
+# * Source License Contributor Agreement. This is important because we      *
+# * fund the project by selling licenses with various terms, and also       *
+# * because the inability to relicense code has caused devastating          *
+# * problems for other Free Software projects (such as KDE and NASM).       *
 # *                                                                         *
-# * This program is distributed in the hope that it will be useful, but     *
-# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
-# * General Public License v2.0 for more details at                         *
-# * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
-# * included with Nmap.                                                     *
+# * The free version of Nmap is distributed in the hope that it will be     *
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+# * indemnification and commercial support are all available through the    *
+# * Npcap OEM program--see https://nmap.org/oem/                            *
 # *                                                                         *
 # ***************************************************************************/
 
 import gobject
 import gtk
 
+
+# Prevent loading PyXML
+import xml
+xml.__path__ = [x for x in xml.__path__ if "_xmlplus" not in x]
+
 from xml.dom import minidom
 
-from zenmapGUI.higwidgets.higboxes import HIGHBox
 from zenmapGUI.higwidgets.higlabels import HIGEntryLabel
 from zenmapGUI.higwidgets.higbuttons import HIGButton
 
 from zenmapGUI.FileChoosers import AllFilesFileChooserDialog
 from zenmapGUI.ProfileHelp import ProfileHelp
 
-from zenmapCore.Paths import Path
 from zenmapCore.NmapOptions import NmapOptions, split_quoted, join_quoted
-import zenmapCore.I18N
-from zenmapCore.UmitLogging import log
-from zenmapGUI.ScriptInterface import *
+import zenmapCore.I18N  # lgtm[py/unused-import]
+from zenmapGUI.ScriptInterface import ScriptInterface
+
 
 def get_option_check_auxiliary_widget(option, ops, check):
     if option in ("-sI", "-b", "--script", "--script-args", "--exclude", "-p",
-        "-D", "-S", "--source-port", "-e", "--ttl", "-iR", "--max-retries",
-        "--host-timeout", "--max-rtt-timeout", "--min-rtt-timeout",
-        "--initial-rtt-timeout", "--max-hostgroup", "--min-hostgroup",
-        "--max-parallelism", "--min-parallelism", "--max-scan-delay",
-        "--scan-delay", "-PA", "-PS", "-PU", "-PO", "-PY"):
+            "-D", "-S", "--source-port", "-e", "--ttl", "-iR", "--max-retries",
+            "--host-timeout", "--max-rtt-timeout", "--min-rtt-timeout",
+            "--initial-rtt-timeout", "--max-hostgroup", "--min-hostgroup",
+            "--max-parallelism", "--min-parallelism", "--max-scan-delay",
+            "--scan-delay", "-PA", "-PS", "-PU", "-PO", "-PY"):
         return OptionEntry(option, ops, check)
     elif option in ("-d", "-v"):
         return OptionLevel(option, ops, check)
     elif option in ("--excludefile", "-iL"):
         return OptionFile(option, ops, check)
     elif option in ("-A", "-O", "-sV", "-n", "-6", "-Pn", "-PE", "-PP", "-PM",
-        "-PB", "-sC", "--script-trace", "-F", "-f", "--packet-trace", "-r",
-        "--traceroute"):
+            "-PB", "-sC", "--script-trace", "-F", "-f", "--packet-trace", "-r",
+            "--traceroute"):
         return None
     elif option in ("",):
         return OptionExtras(option, ops, check)
     else:
         assert False, "Unknown option %s" % option
 
+
 class OptionEntry(gtk.Entry):
     def __init__(self, option, ops, check):
         gtk.Entry.__init__(self)
@@ -153,6 +129,7 @@ class OptionEntry(gtk.Entry):
         self.check.set_active(True)
         self.ops[self.option] = self.get_text().decode("UTF-8")
 
+
 class OptionExtras(gtk.Entry):
     def __init__(self, option, ops, check):
         gtk.Entry.__init__(self)
@@ -180,6 +157,7 @@ class OptionExtras(gtk.Entry):
         self.check.set_active(True)
         self.ops.extras = [self.get_text().decode("UTF-8")]
 
+
 class OptionLevel(gtk.SpinButton):
     def __init__(self, option, ops, check):
         gtk.SpinButton.__init__(self, gtk.Adjustment(0, 0, 10, 1), 0.0, 0)
@@ -209,6 +187,7 @@ class OptionLevel(gtk.SpinButton):
         self.check.set_active(True)
         self.ops[self.option] = int(self.get_adjustment().get_value())
 
+
 class OptionFile(gtk.HBox):
     __gsignals__ = {
         "changed": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ())
@@ -223,7 +202,7 @@ class OptionFile(gtk.HBox):
 
         self.entry = gtk.Entry()
         self.pack_start(self.entry, True, True)
-        button = HIGButton(stock = gtk.STOCK_OPEN)
+        button = HIGButton(stock=gtk.STOCK_OPEN)
         self.pack_start(button, False)
 
         button.connect("clicked", self.clicked_cb)
@@ -257,6 +236,7 @@ class OptionFile(gtk.HBox):
             self.entry.set_text(dialog.get_filename())
         dialog.destroy()
 
+
 class TargetEntry(gtk.Entry):
     def __init__(self, ops):
         gtk.Entry.__init__(self)
@@ -267,17 +247,18 @@ class TargetEntry(gtk.Entry):
     def update(self):
         self.set_text(u" ".join(self.ops.target_specs))
 
-    def changed_cb(self, eidget):
+    def changed_cb(self, widget):
         self.ops.target_specs = self.get_targets()
 
     def get_targets(self):
         return split_quoted(self.get_text().decode("UTF-8"))
 
+
 class OptionTab(object):
     def __init__(self, root_tab, ops, update_command, help_buf):
-        actions = {'target':self.__parse_target,
-                   'option_list':self.__parse_option_list,
-                   'option_check':self.__parse_option_check}
+        actions = {'target': self.__parse_target,
+                   'option_list': self.__parse_option_list,
+                   'option_check': self.__parse_option_check}
 
         self.ops = ops
         self.update_command = update_command
@@ -287,13 +268,11 @@ class OptionTab(object):
         self.notscripttab = False  # assume every tab is scripting tab
         self.widgets_list = []
         for option_element in root_tab.childNodes:
-            try:option_element.tagName
-            except:pass
-            else:
-                if option_element.tagName in actions.keys():
-                    parse_func = actions[option_element.tagName]
-                    widget = parse_func(option_element)
-                    self.widgets_list.append(widget)
+            if (hasattr(option_element, "tagName") and
+                    option_element.tagName in actions.keys()):
+                parse_func = actions[option_element.tagName]
+                widget = parse_func(option_element)
+                self.widgets_list.append(widget)
 
     def __parse_target(self, target_element):
         label = _(target_element.getAttribute(u'label'))
@@ -305,7 +284,8 @@ class OptionTab(object):
     def __parse_option_list(self, option_list_element):
         children = option_list_element.getElementsByTagName(u'option')
 
-        label_widget = HIGEntryLabel(_(option_list_element.getAttribute(u'label')))
+        label_widget = HIGEntryLabel(
+                _(option_list_element.getAttribute(u'label')))
         option_list_widget = OptionList(self.ops)
 
         for child in children:
@@ -314,8 +294,10 @@ class OptionTab(object):
             label = _(child.getAttribute(u'label'))
             option_list_widget.append(option, argument, label)
             self.profilehelp.add_label(option, label)
-            self.profilehelp.add_shortdesc(option, _(child.getAttribute(u'short_desc')))
-            self.profilehelp.add_example(option, child.getAttribute(u'example'))
+            self.profilehelp.add_shortdesc(
+                    option, _(child.getAttribute(u'short_desc')))
+            self.profilehelp.add_example(
+                    option, child.getAttribute(u'example'))
 
         option_list_widget.update()
 
@@ -324,7 +306,6 @@ class OptionTab(object):
         return label_widget, option_list_widget
 
     def __parse_option_check(self, option_check):
-        arg_type = option_check.getAttribute(u'arg_type')
         option = option_check.getAttribute(u'option')
         label = _(option_check.getAttribute(u'label'))
         short_desc = _(option_check.getAttribute(u'short_desc'))
@@ -335,10 +316,12 @@ class OptionTab(object):
         self.profilehelp.add_example(option, example)
 
         check = OptionCheck(option, label)
-        auxiliary_widget = get_option_check_auxiliary_widget(option, self.ops, check)
+        auxiliary_widget = get_option_check_auxiliary_widget(
+                option, self.ops, check)
         if auxiliary_widget is not None:
             auxiliary_widget.connect("changed", self.update_auxiliary_widget)
-            auxiliary_widget.connect('enter-notify-event', self.enter_notify_event_cb, option)
+            auxiliary_widget.connect(
+                    'enter-notify-event', self.enter_notify_event_cb, option)
         else:
             check.set_active(not not self.ops[option])
 
@@ -347,14 +330,14 @@ class OptionTab(object):
 
         return check, auxiliary_widget
 
-    def fill_table(self, table, expand_fill = True):
+    def fill_table(self, table, expand_fill=True):
         yopt = (0, gtk.EXPAND | gtk.FILL)[expand_fill]
         for y, widget in enumerate(self.widgets_list):
-            if widget[1] == None:
-                table.attach(widget[0], 0, 2, y, y+1, yoptions=yopt)
+            if widget[1] is None:
+                table.attach(widget[0], 0, 2, y, y + 1, yoptions=yopt)
             else:
-                table.attach(widget[0], 0, 1, y, y+1, yoptions=yopt)
-                table.attach(widget[1], 1, 2, y, y+1, yoptions=yopt)
+                table.attach(widget[0], 0, 1, y, y + 1, yoptions=yopt)
+                table.attach(widget[1], 1, 2, y, y + 1, yoptions=yopt)
 
     def update_auxiliary_widget(self, auxiliary_widget):
         self.update_command()
@@ -412,6 +395,7 @@ class OptionTab(object):
     def enter_notify_event_cb(self, event, widget, option):
         self.show_help_for_option(option)
 
+
 class OptionBuilder(object):
     def __init__(self, xml_file, ops, update_func, help_buf):
         """
@@ -429,7 +413,6 @@ class OptionBuilder(object):
 
         self.root_tag = "interface"
 
-
         self.xml = self.xml.getElementsByTagName(self.root_tag)[0]
 
         self.groups = self.__parse_groups()
@@ -448,19 +431,20 @@ class OptionBuilder(object):
         return dic
 
     def __parse_groups(self):
-        return [g_name.getAttribute(u'name') for g_name in \
-                  self.xml.getElementsByTagName(u'groups')[0].\
-                  getElementsByTagName(u'group')]
+        return [g_name.getAttribute(u'name') for g_name in
+                self.xml.getElementsByTagName(u'groups')[0].getElementsByTagName(u'group')]  # noqa
 
     def __parse_tabs(self):
         dic = {}
         for tab_name in self.groups:
             if tab_name != "Scripting":
-                dic[tab_name] = OptionTab(self.xml.getElementsByTagName(tab_name)[0],
-                                      self.ops, self.update_func, self.help_buf)
+                dic[tab_name] = OptionTab(
+                        self.xml.getElementsByTagName(tab_name)[0], self.ops,
+                        self.update_func, self.help_buf)
                 dic[tab_name].notscripttab = True
             else:
-                dic[tab_name] =ScriptInterface(None,self.ops, self.update_func, self.help_buf)
+                dic[tab_name] = ScriptInterface(
+                        None, self.ops, self.update_func, self.help_buf)
         return dic
 
 
@@ -484,7 +468,8 @@ class OptionList(gtk.ComboBox):
             opt, arg = row[0], row[1]
             if opt == "":
                 continue
-            if (not arg and self.ops[opt]) or (arg and str(self.ops[opt]) == arg):
+            if ((not arg and self.ops[opt]) or
+                    (arg and str(self.ops[opt]) == arg)):
                 selected = i
         self.set_active(selected)
 
@@ -501,6 +486,7 @@ class OptionList(gtk.ComboBox):
         self.list.append([option, argument, opt])
         self.options.append(option)
 
+
 class OptionCheck(gtk.CheckButton):
     def __init__(self, option, label):
         opt = label
