@@ -3,130 +3,68 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
-# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
-# * modify, and redistribute this software under certain conditions.  If    *
-# * you wish to embed Nmap technology into proprietary software, we sell    *
-# * alternative licenses (contact sales@insecure.com).  Dozens of software  *
-# * vendors already license Nmap technology such as host discovery, port    *
-# * scanning, OS detection, version detection, and the Nmap Scripting       *
-# * Engine.                                                                 *
+# * The Nmap Security Scanner is (C) 1996-2022 Nmap Software LLC ("The Nmap *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derivative works",  *
-# * yet it does not provide a detailed definition of that term.  To avoid   *
-# * misunderstandings, we interpret that term as broadly as copyright law   *
-# * allows.  For example, we consider an application to constitute a        *
-# * derivative work for the purpose of this license if it does any of the   *
-# * following with any software or content covered by this license          *
-# * ("Covered Software"):                                                   *
+# * This program is distributed under the terms of the Nmap Public Source   *
+# * License (NPSL). The exact license text applying to a particular Nmap    *
+# * release or source code control revision is contained in the LICENSE     *
+# * file distributed with that version of Nmap or source code control       *
+# * revision. More Nmap copyright/legal information is available from       *
+# * https://nmap.org/book/man-legal.html, and further information on the    *
+# * NPSL license itself can be found at https://nmap.org/npsl/ . This       *
+# * header summarizes some key points from the Nmap license, but is no      *
+# * substitute for the actual license text.                                 *
 # *                                                                         *
-# * o Integrates source code from Covered Software.                         *
+# * Nmap is generally free for end users to download and use themselves,    *
+# * including commercial use. It is available from https://nmap.org.        *
 # *                                                                         *
-# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
-# * or nmap-service-probes.                                                 *
+# * The Nmap license generally prohibits companies from using and           *
+# * redistributing Nmap in commercial products, but we sell a special Nmap  *
+# * OEM Edition with a more permissive license and special features for     *
+# * this purpose. See https://nmap.org/oem/                                 *
 # *                                                                         *
-# * o Is designed specifically to execute Covered Software and parse the    *
-# * results (as opposed to typical shell or execution-menu apps, which will *
-# * execute anything you tell them to).                                     *
+# * If you have received a written Nmap license agreement or contract       *
+# * stating terms other than these (such as an Nmap OEM license), you may   *
+# * choose to use and redistribute Nmap under those terms instead.          *
 # *                                                                         *
-# * o Includes Covered Software in a proprietary executable installer.  The *
-# * installers produced by InstallShield are an example of this.  Including *
-# * Nmap with other software in compressed or archival form does not        *
-# * trigger this provision, provided appropriate open source decompression  *
-# * or de-archiving software is widely available for no charge.  For the    *
-# * purposes of this license, an installer is considered to include Covered *
-# * Software even if it actually retrieves a copy of Covered Software from  *
-# * another source during runtime (such as by downloading it from the       *
-# * Internet).                                                              *
-# *                                                                         *
-# * o Links (statically or dynamically) to a library which does any of the  *
-# * above.                                                                  *
-# *                                                                         *
-# * o Executes a helper program, module, or script to do any of the above.  *
-# *                                                                         *
-# * This list is not exclusive, but is meant to clarify our interpretation  *
-# * of derived works with some common examples.  Other people may interpret *
-# * the plain GPL differently, so we consider this a special exception to   *
-# * the GPL that we apply to Covered Software.  Works which meet any of     *
-# * these conditions must conform to all of the terms of this license,      *
-# * particularly including the GPL Section 3 requirements of providing      *
-# * source code and allowing free redistribution of the work as a whole.    *
-# *                                                                         *
-# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two.                                  *
-# *                                                                         *
-# * Any redistribution of Covered Software, including any derived works,    *
-# * must obey and carry forward all of the terms of this license, including *
-# * obeying all GPL rules and restrictions.  For example, source code of    *
-# * the whole work must be provided and free redistribution must be         *
-# * allowed.  All GPL references to "this License", are to be treated as    *
-# * including the special and conditions of the license text as well.       *
-# *                                                                         *
-# * Because this license imposes special exceptions to the GPL, Covered     *
-# * Work may not be combined (even as part of a larger work) with plain GPL *
-# * software.  The terms, conditions, and exceptions of this license must   *
-# * be included as well.  This license is incompatible with some other open *
-# * source licenses as well.  In some cases we can relicense portions of    *
-# * Nmap or grant special permissions to use it in other open source        *
-# * software.  Please contact fyodor@nmap.org with any such requests.       *
-# * Similarly, we don't incorporate incompatible open source software into  *
-# * Covered Software without special permission from the copyright holders. *
-# *                                                                         *
-# * If you have any questions about the licensing restrictions on using     *
-# * Nmap in other works, are happy to help.  As mentioned above, we also    *
-# * offer alternative license to integrate Nmap into proprietary            *
-# * applications and appliances.  These contracts have been sold to dozens  *
-# * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@insecure.com for     *
-# * further information.                                                    *
-# *                                                                         *
-# * If you received these files with a written license agreement or         *
-# * contract stating terms other than the terms above, then that            *
-# * alternative license agreement takes precedence over these comments.     *
+# * The official Nmap Windows builds include the Npcap software             *
+# * (https://npcap.com) for packet capture and transmission. It is under    *
+# * separate license terms which forbid redistribution without special      *
+# * permission. So the official Nmap Windows builds may not be              *
+# * redistributed without special permission (such as an Nmap OEM           *
+# * license).                                                               *
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
-# * and add new features.  You are highly encouraged to send your changes   *
-# * to the dev@nmap.org mailing list for possible incorporation into the    *
-# * main distribution.  By sending these changes to Fyodor or one of the    *
-# * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
-# * occasionally relicense the code to third parties as discussed above.    *
-# * If you wish to specify special license conditions of your               *
-# * contributions, just say so when you send them.                          *
+# * and add new features.  You are highly encouraged to submit your         *
+# * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+# * for possible incorporation into the main distribution. Unless you       *
+# * specify otherwise, it is understood that you are offering us very       *
+# * broad rights to use your submissions as described in the Nmap Public    *
+# * Source License Contributor Agreement. This is important because we      *
+# * fund the project by selling licenses with various terms, and also       *
+# * because the inability to relicense code has caused devastating          *
+# * problems for other Free Software projects (such as KDE and NASM).       *
 # *                                                                         *
-# * This program is distributed in the hope that it will be useful, but     *
-# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
-# * license file for more details (it's in a COPYING file included with     *
-# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
+# * The free version of Nmap is distributed in the hope that it will be     *
+# * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+# * indemnification and commercial support are all available through the    *
+# * Npcap OEM program--see https://nmap.org/oem/                            *
 # *                                                                         *
 # ***************************************************************************/
 
 import gtk
 
-from zenmapGUI.higwidgets.higboxes import HIGVBox, HIGHBox
-from zenmapGUI.higwidgets.higtables import HIGTable
+from zenmapGUI.higwidgets.higboxes import HIGVBox
 
 from zenmapCore.UmitLogging import log
-import zenmapCore.I18N
+import zenmapCore.I18N  # lgtm[py/unused-import]
+
 
 def findout_service_icon(port_info):
     if port_info["port_state"] in ["open", "open|filtered"]:
@@ -134,6 +72,7 @@ def findout_service_icon(port_info):
     else:
         return gtk.STOCK_NO
 
+
 def get_version_string(d):
     """Get a human-readable version string from the dict d. The keys used in d
     are "service_product", "service_version", and "service_extrainfo" (all are
@@ -148,38 +87,45 @@ def get_version_string(d):
         result.append("(" + d["service_extrainfo"] + ")")
     return " ".join(result)
 
+
 def get_addrs(host):
     if host is None:
         return None
     return host.get_addrs_for_sort()
 
+
 def cmp_addrs(host_a, host_b):
     return cmp(get_addrs(host_a), get_addrs(host_b))
 
+
 def cmp_port_list_addr(model, iter_a, iter_b):
     host_a = model.get_value(iter_a, 0)
     host_b = model.get_value(iter_b, 0)
     return cmp_addrs(host_a, host_b)
 
+
 def cmp_port_tree_addr(model, iter_a, iter_b):
     host_a = model.get_value(iter_a, 0)
     host_b = model.get_value(iter_b, 0)
     return cmp_addrs(host_a, host_b)
 
+
 def cmp_host_list_addr(model, iter_a, iter_b):
     host_a = model.get_value(iter_a, 2)
     host_b = model.get_value(iter_b, 2)
     return cmp_addrs(host_a, host_b)
 
+
 def cmp_host_tree_addr(model, iter_a, iter_b):
     host_a = model.get_value(iter_a, 2)
     host_b = model.get_value(iter_b, 2)
     return cmp_addrs(host_a, host_b)
 
+
 class ScanOpenPortsPage(gtk.ScrolledWindow):
     def __init__(self):
         gtk.ScrolledWindow.__init__(self)
-        self.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
+        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 
         self.__create_widgets()
 
@@ -188,6 +134,7 @@ class ScanOpenPortsPage(gtk.ScrolledWindow):
     def __create_widgets(self):
         self.host = HostOpenPorts()
 
+
 class HostOpenPorts(HIGVBox):
     def __init__(self):
         HIGVBox.__init__(self)
@@ -203,8 +150,10 @@ class HostOpenPorts(HIGVBox):
         # host hostname icon port protocol state service version
         # The hostname column is shown only when more than one host is selected
         # (hence port_tree not port_list is used).
-        self.port_list = gtk.ListStore(object, str, str, int, str, str, str, str)
-        self.port_tree = gtk.TreeStore(object, str, str, int, str, str, str, str)
+        self.port_list = gtk.ListStore(
+                object, str, str, int, str, str, str, str)
+        self.port_tree = gtk.TreeStore(
+                object, str, str, int, str, str, str, str)
 
         self.port_list.set_sort_func(1000, cmp_port_list_addr)
         self.port_list.set_sort_column_id(1000, gtk.SORT_ASCENDING)
@@ -229,8 +178,10 @@ class HostOpenPorts(HIGVBox):
         # service icon host hostname port protocol state version
         # service is shown only when more than one service is selected (hence
         # host_tree not host_list is used).
-        self.host_list = gtk.ListStore(str, str, object, str, int, str, str, str)
-        self.host_tree = gtk.TreeStore(str, str, object, str, int, str, str, str)
+        self.host_list = gtk.ListStore(
+                str, str, object, str, int, str, str, str)
+        self.host_tree = gtk.TreeStore(
+                str, str, object, str, int, str, str, str)
 
         self.host_list.set_sort_func(1000, cmp_host_list_addr)
         self.host_list.set_sort_column_id(1000, gtk.SORT_ASCENDING)
@@ -285,7 +236,8 @@ class HostOpenPorts(HIGVBox):
         self.host_columns['state'].pack_start(self.cell_port, True)
 
         self.host_columns['service'].set_attributes(self.cell_port, text=0)
-        self.host_columns['icon'].set_attributes(self.cell_host_icon, stock_id=1)
+        self.host_columns['icon'].set_attributes(
+                self.cell_host_icon, stock_id=1)
         self.host_columns['hostname'].set_attributes(self.cell_port, text=3)
         self.host_columns['port_number'].set_attributes(self.cell_port, text=4)
         self.host_columns['protocol'].set_attributes(self.cell_port, text=5)
@@ -294,7 +246,8 @@ class HostOpenPorts(HIGVBox):
 
         self.host_columns['service'].set_visible(False)
 
-        self.scroll_ports_hosts.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.scroll_ports_hosts.set_policy(
+                gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 
     def _set_port_list(self):
         self.port_view.set_enable_search(True)
@@ -315,7 +268,6 @@ class HostOpenPorts(HIGVBox):
             self.port_columns[k].set_reorderable(True)
             self.port_columns[k].set_resizable(True)
 
-
         self.port_columns['icon'].set_min_width(35)
 
         self.port_columns['hostname'].set_sort_column_id(1000)
@@ -344,8 +296,8 @@ class HostOpenPorts(HIGVBox):
 
         self.port_columns['hostname'].set_visible(False)
 
-        self.scroll_ports_hosts.set_policy(gtk.POLICY_AUTOMATIC,\
-                                          gtk.POLICY_AUTOMATIC)
+        self.scroll_ports_hosts.set_policy(
+                gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 
     def port_mode(self):
         child = self.scroll_ports_hosts.get_child()
@@ -368,10 +320,14 @@ class HostOpenPorts(HIGVBox):
     def freeze(self):
         """Freeze notifications and sorting to make adding lots of elements to
         the model faster."""
-        self.frozen_host_list_sort_column_id = self.host_list.get_sort_column_id()
-        self.frozen_host_tree_sort_column_id = self.host_tree.get_sort_column_id()
-        self.frozen_port_list_sort_column_id = self.port_list.get_sort_column_id()
-        self.frozen_port_tree_sort_column_id = self.port_tree.get_sort_column_id()
+        self.frozen_host_list_sort_column_id = \
+                self.host_list.get_sort_column_id()
+        self.frozen_host_tree_sort_column_id = \
+                self.host_tree.get_sort_column_id()
+        self.frozen_port_list_sort_column_id = \
+                self.port_list.get_sort_column_id()
+        self.frozen_port_tree_sort_column_id = \
+                self.port_tree.get_sort_column_id()
         self.host_list.set_default_sort_func(lambda *args: -1)
         self.host_tree.set_default_sort_func(lambda *args: -1)
         self.port_list.set_default_sort_func(lambda *args: -1)
@@ -387,13 +343,17 @@ class HostOpenPorts(HIGVBox):
         """Restore notifications and sorting (after making changes to the
         model)."""
         if self.frozen_host_list_sort_column_id != (None, None):
-            self.host_list.set_sort_column_id(*self.frozen_host_list_sort_column_id)
+            self.host_list.set_sort_column_id(
+                    *self.frozen_host_list_sort_column_id)
         if self.frozen_host_tree_sort_column_id != (None, None):
-            self.host_tree.set_sort_column_id(*self.frozen_host_tree_sort_column_id)
+            self.host_tree.set_sort_column_id(
+                    *self.frozen_host_tree_sort_column_id)
         if self.frozen_port_list_sort_column_id != (None, None):
-            self.port_list.set_sort_column_id(*self.frozen_port_list_sort_column_id)
+            self.port_list.set_sort_column_id(
+                    *self.frozen_port_list_sort_column_id)
         if self.frozen_port_tree_sort_column_id != (None, None):
-            self.port_tree.set_sort_column_id(*self.frozen_port_tree_sort_column_id)
+            self.port_tree.set_sort_column_id(
+                    *self.frozen_port_tree_sort_column_id)
         self.host_view.set_model(self.frozen_host_view_model)
         self.port_view.set_model(self.frozen_port_view_model)
         self.host_view.thaw_child_notify()
@@ -414,7 +374,8 @@ class HostOpenPorts(HIGVBox):
         self.host_list.append(entry)
 
     def add_to_port_tree(self, host):
-        parent = self.port_tree.append(None, [host, host.get_hostname(), None, 0,'','','',''])
+        parent = self.port_tree.append(
+                None, [host, host.get_hostname(), None, 0, '', '', '', ''])
         for p in host.get_ports():
             self.port_tree.append(parent,
                 [None, '', findout_service_icon(p), int(p.get('portid', "0")),
@@ -422,12 +383,21 @@ class HostOpenPorts(HIGVBox):
                 p.get('service_name', _("Unknown")), get_version_string(p)])
 
     def add_to_host_tree(self, service_name, ports):
-        parent = self.host_tree.append(None, [service_name, '', None, '', 0, '', '', ''])
+        parent = self.host_tree.append(
+                None, [service_name, '', None, '', 0, '', '', ''])
         for p in ports:
             self.host_tree.append(parent,
-                ['', findout_service_icon(p), p["host"], p["host"].get_hostname(),
-                int(p.get('portid', "0")), p.get('protocol', ""),
-                p.get('port_state', _("Unknown")), get_version_string(p)])
+                    [
+                        '',
+                        findout_service_icon(p),
+                        p["host"],
+                        p["host"].get_hostname(),
+                        int(p.get('portid', "0")),
+                        p.get('protocol', ""),
+                        p.get('port_state', _("Unknown")),
+                        get_version_string(p)
+                    ]
+                )
 
     def switch_port_to_list_store(self):
         if self.port_view.get_model() != self.port_list:
